# !/usr/bin/env python
# -*- coding: utf-8 -*-

"""多尺度模板匹配.

对用户提供的调节参数:
    1. threshod: 筛选阈值，默认为0.8
    2. rgb: 彩色三通道,进行彩色权识别.
    3. scale_max: 多尺度模板匹配最大范围，增大可适应更小UI
    4. scale_step: 多尺度模板匹配搜索比例步长，减小可适应更小UI
"""
from __future__ import division
from __future__ import print_function

import cv2
import time

from airtestProject.airtest.utils.logger import get_logger
from airtestProject.airtest.aircv.error import TemplateInputError
from airtestProject.airtest import aircv
from .utils import generate_result, check_source_larger_than_search, img_mat_rgb_2_gray, print_run_time
from .cal_confidence import cal_rgb_confidence, cal_ccoeff_confidence

LOGGING = get_logger(__name__)


class MultiScaleTemplateMatching(object):
    """多尺度模板匹配."""

    METHOD_NAME = "MSTemplate"

    def __init__(self, im_search, im_source, threshold=0.8, rgb=True, record_pos=None, resolution=(), scale_max=800, scale_step=0.005):
        self.im_source = im_source
        self.im_search = im_search
        self.threshold = threshold
        self.rgb = rgb
        self.record_pos = record_pos
        self.resolution = resolution
        self.scale_max = scale_max
        self.scale_step = scale_step

    def find_all_results(self):
        raise NotImplementedError

    @print_run_time
    def find_best_result(self):
        """函数功能：找到最优结果."""
        # 第一步：校验图像输入
        check_source_larger_than_search(self.im_source, self.im_search)

        # 第二步：计算模板匹配的结果矩阵res
        s_gray, i_gray = img_mat_rgb_2_gray(
            self.im_search), img_mat_rgb_2_gray(self.im_source)
        confidence, max_loc, w, h, _ = self.multi_scale_search(
            i_gray, s_gray, ratio_min=0.01, ratio_max=0.99, src_max=self.scale_max, step=self.scale_step, threshold=self.threshold)

        # 求取识别位置: 目标中心 + 目标区域:
        middle_point, rectangle = self._get_target_rectangle(max_loc, w, h)
        best_match = generate_result(middle_point, rectangle, confidence)
        LOGGING.debug("[%s] threshold=%s, result=%s" %
                      (self.METHOD_NAME, self.threshold, best_match))

        return best_match if confidence >= self.threshold else None

    def _get_confidence_from_matrix(self, max_loc, w, h):
        """根据结果矩阵求出confidence."""
        sch_h, sch_w = self.im_search.shape[0], self.im_search.shape[1]
        if self.rgb:
            # 如果有颜色校验,对目标区域进行BGR三通道校验:
            img_crop = self.im_source[max_loc[1]:max_loc[1] + h, max_loc[0]: max_loc[0] + w]
            confidence = cal_rgb_confidence(
                cv2.resize(img_crop, (sch_w, sch_h)), self.im_search)
        else:
            img_crop = self.im_source[max_loc[1]:max_loc[1] + h, max_loc[0]: max_loc[0] + w]
            confidence = cal_ccoeff_confidence(
                cv2.resize(img_crop, (sch_w, sch_h)), self.im_search)

        return confidence

    def _get_target_rectangle(self, left_top_pos, w, h):
        """根据左上角点和宽高求出目标区域."""
        x_min, y_min = left_top_pos
        # 中心位置的坐标:
        x_middle, y_middle = int(x_min + w / 2), int(y_min + h / 2)
        # 左下(min,max)->右下(max,max)->右上(max,min)
        left_bottom_pos, right_bottom_pos = (
            x_min, y_min + h), (x_min + w, y_min + h)
        right_top_pos = (x_min + w, y_min)
        # 点击位置:
        middle_point = (x_middle, y_middle)
        # 识别目标区域: 点序:左上->左下->右下->右上, 左上(min,min)右下(max,max)
        rectangle = (left_top_pos, left_bottom_pos,
                     right_bottom_pos, right_top_pos)

        return middle_point, rectangle

    @staticmethod
    def _resize_by_ratio(src, templ, ratio=1.0, templ_min=10, src_max=800):
        """根据模板相对屏幕的长边 按比例缩放屏幕"""
        # 截屏最大尺寸限制
        sr = min(src_max/max(src.shape),1.0)
        src = cv2.resize(src, (int(src.shape[1]*sr), int(src.shape[0]*sr)))
        # 截图尺寸缩放
        h, w = src.shape[0], src.shape[1]
        th, tw = templ.shape[0], templ.shape[1]
        if th/h >= tw/w:
            tr = (h*ratio)/th
        else:
            tr = (w*ratio)/tw
        templ = cv2.resize(templ, (max(int(tw*tr), 1), max(int(th*tr), 1)))
        return src, templ, tr, sr

    @staticmethod
    def _org_size(max_loc, w, h, tr, sr):
        """获取原始比例的框"""
        max_loc = (int((max_loc[0]/sr)), int((max_loc[1]/sr)))
        w, h = int((w/sr)), int((h/sr))
        return max_loc, w, h

    def multi_scale_search(self, org_src, org_templ, templ_min=10, src_max=800, ratio_min=0.01, 
                            ratio_max=0.99, step=0.01, threshold=0.8, time_out=3.0):
        """多尺度模板匹配"""
        mmax_val = 0
        max_info = None
        r = ratio_min
        t = time.time()
        while r <= ratio_max:
            src, templ, tr, sr = self._resize_by_ratio(
                org_src.copy(), org_templ.copy(), r, src_max=src_max)
            if min(templ.shape) > templ_min:
                src[0,0] = templ[0,0] = 0
                src[0,1] = templ[0,1] = 255
                result = cv2.matchTemplate(src, templ, cv2.TM_CCOEFF_NORMED)
                min_val, max_val, min_loc, max_loc = cv2.minMaxLoc(result)
                h, w = templ.shape
                if mmax_val < max_val:
                    mmax_val = max_val
                    max_info = (r, max_val, max_loc, w, h, tr, sr)
                # print((r, max_val, max_loc, w, h, tr, sr))
                time_cost = time.time() - t
                if time_cost>time_out and max_val >= threshold:
                    omax_loc, ow, oh = self._org_size(max_loc, w, h, tr, sr)
                    confidence = self._get_confidence_from_matrix(omax_loc, ow, oh)
                    if confidence >= threshold:
                        return confidence, omax_loc, ow, oh, r
            r += step
        if max_info is None:
            return 0, (0, 0), 0, 0, 0
        max_r, max_val, max_loc, w, h, tr, sr = max_info
        omax_loc, ow, oh = self._org_size(max_loc, w, h, tr, sr)
        confidence = self._get_confidence_from_matrix(omax_loc, ow, oh)
        return confidence, omax_loc, ow, oh, max_r


class MultiScaleTemplateMatchingPre(MultiScaleTemplateMatching):
    """基于截图预设条件的多尺度模板匹配."""

    METHOD_NAME = "MSTemplatePre"
    DEVIATION = 150

    @print_run_time
    def find_best_result(self):
        """函数功能：找到最优结果."""
        if self.resolution!=():
            # 第一步：校验图像输入
            check_source_larger_than_search(self.im_source, self.im_search)
            if self.resolution[0]<self.im_search.shape[1] or self.resolution[1]<self.im_search.shape[0]:
                raise TemplateInputError("error: resolution is too small.")
            # 第二步：计算模板匹配的结果矩阵res
            if not self.record_pos is None:
                area, self.resolution = self._get_area_scope(self.im_source, self.im_search, self.record_pos, self.resolution)
                self.im_source = aircv.crop_image(self.im_source, area)
                check_source_larger_than_search(self.im_source, self.im_search)
            r_min, r_max = self._get_ratio_scope(
                self.im_source, self.im_search, self.resolution)
            s_gray, i_gray = img_mat_rgb_2_gray(self.im_search), img_mat_rgb_2_gray(self.im_source)
            confidence, max_loc, w, h, _ = self.multi_scale_search(
                    i_gray, s_gray, ratio_min=r_min, ratio_max=r_max, step=self.scale_step, 
                    threshold=self.threshold, time_out=1.0)
            if not self.record_pos is None:
                max_loc = (max_loc[0] + area[0], max_loc[1] + area[1])
            
            # 求取识别位置: 目标中心 + 目标区域:
            middle_point, rectangle = self._get_target_rectangle(max_loc, w, h)
            best_match = generate_result(middle_point, rectangle, confidence)
            LOGGING.debug("[%s] threshold=%s, result=%s" %
                        (self.METHOD_NAME, self.threshold, best_match))

            return best_match if confidence >= self.threshold else None
        else:
            return None

    def _get_ratio_scope(self, src, templ, resolution):
        """预测缩放比的上下限."""
        H, W = src.shape[0], src.shape[1]
        th, tw = templ.shape[0], templ.shape[1]
        w, h = resolution
        rmin = min(H/h, W/w)  # 新旧模板比下限
        rmax = max(H/h, W/w)  # 新旧模板比上限
        ratio = max(th/H, tw/W)  # 小图大图比
        r_min = ratio*rmin
        r_max = ratio*rmax
        return max(r_min, self.scale_step), min(r_max, 0.99)

    def get_predict_point(self, record_pos, screen_resolution):
        """预测缩放后的点击位置点."""
        delta_x, delta_y = record_pos
        _w, _h = screen_resolution
        target_x = delta_x * _w + _w * 0.5
        target_y = delta_y * _w + _h * 0.5
        return target_x, target_y

    def _get_area_scope(self, src, templ, record_pos, resolution):
        """预测搜索区域."""
        H, W = src.shape[0], src.shape[1]
        th, tw = templ.shape[0], templ.shape[1]
        w, h = resolution
        x, y = self.get_predict_point(record_pos, (W, H))
        predict_x_radius = max(int(tw * W / (w)), self.DEVIATION)
        predict_y_radius = max(int(th * H / (h)), self.DEVIATION)
        area = (
            max(x - predict_x_radius, 0),
            max(y - predict_y_radius, 0),
            min(x + predict_x_radius, W),   
            min(y + predict_y_radius, H),
            )
        return area, (w*(area[3]-area[1])/W, h*(area[2]-area[0])/H)




import traceback
import warnings
from .constant import YOSEMITE_APK, YOSEMITE_PACKAGE
from airtestProject.airtest.utils.snippet import on_method_ready
from airtestProject.airtest.utils.apkparser import APK
from airtestProject.airtest.utils.logger import get_logger
LOGGING = get_logger(__name__)


class Yosemite(object):
    """Wrapper class of Yosemite.apk, used by javacap/recorder/yosemite_ime."""

    def __init__(self, adb):
        self.adb = adb

    def install_or_upgrade(self):
        """
        Install or update the Yosemite.apk file on the device

        Returns:
            None

        """
        self._install_apk_upgrade(YOSEMITE_APK, YOSEMITE_PACKAGE)

    def _install_apk_upgrade(self, apk_path, package):
        """
        Install or update the `.apk` file on the device

        Args:
            apk_path: full path `.apk` file
            package: package name

        Returns:
            None

        """
        apk_version = int(APK(apk_path).androidversion_code)
        installed_version = self.adb.get_package_version(package)
        if installed_version is None or apk_version > int(installed_version):
            LOGGING.info(
                "local version code is {}, installed version code is {}".format(apk_version, installed_version))
            try:
                self.adb.pm_install(apk_path, replace=True, install_options=["-t"])
            except:
                if installed_version is None:
                    raise
                # If the installation fails, but the phone has an old version, do not force the installation
                print(traceback.format_exc())
                warnings.warn("Yosemite.apk update failed, please try to reinstall manually(airtest/core/android/static/apks/Yosemite.apk).")

    @on_method_ready('install_or_upgrade')
    def get_ready(self):
        pass

    def uninstall(self):
        """
        Uninstall `Yosemite.apk` application from the device

        Returns:
            None

        """
        self.adb.uninstall_app(YOSEMITE_PACKAGE)



# encoding=utf-8
from six import with_metaclass


class MetaDevice(type):

    REGISTRY = {}

    def __new__(meta, name, bases, class_dict):
        cls = type.__new__(meta, name, bases, class_dict)
        meta.REGISTRY[name] = cls
        return cls


class Device(with_metaclass(MetaDevice, object)):
    """base class for test device"""

    def __init__(self):
        super(Device, self).__init__()

    def to_json(self):
        try:
            uuid = repr(self.uuid)
        except:
            uuid = None
        return f"<{self.__class__.__name__} {uuid}>"

    @property
    def uuid(self):
        self._raise_not_implemented_error()

    def shell(self, *args, **kwargs):
        self._raise_not_implemented_error()

    def snapshot(self, *args, **kwargs):
        self._raise_not_implemented_error()

    def touch(self, target, **kwargs):
        self._raise_not_implemented_error()

    def double_click(self, target):
        raise NotImplementedError

    def swipe(self, t1, t2, **kwargs):
        self._raise_not_implemented_error()

    def keyevent(self, key, **kwargs):
        self._raise_not_implemented_error()

    def text(self, text, enter=True):
        self._raise_not_implemented_error()

    def start_app(self, package, **kwargs):
        self._raise_not_implemented_error()

    def stop_app(self, package):
        self._raise_not_implemented_error()

    def clear_app(self, package):
        self._raise_not_implemented_error()

    def list_app(self, **kwargs):
        self._raise_not_implemented_error()

    def install_app(self, uri, **kwargs):
        self._raise_not_implemented_error()

    def uninstall_app(self, package):
        self._raise_not_implemented_error()

    def get_current_resolution(self):
        self._raise_not_implemented_error()

    def get_render_resolution(self):
        self._raise_not_implemented_error()

    def get_ip_address(self):
        self._raise_not_implemented_error()

    def set_clipboard(self, text):
        self._raise_not_implemented_error()

    def get_clipboard(self):
        self._raise_not_implemented_error()

    def paste(self):
        self.text(self.get_clipboard())

    def _raise_not_implemented_error(self):
        platform = self.__class__.__name__
        raise NotImplementedError("Method not implemented on %s" % platform)

    def disconnect(self):
        pass



"""
日志工具
"""
import random
import subprocess

import loguru
from typing import Callable
from airtestProject.airtest.core import api as air

import functools
import time
import sys
import os

import logging

from airtestProject.airtest.core.helper import tag_listener
from airtestProject.commons.utils.tools import log_error_plus

# 设置airtest日志等级，debug会输出很多日志
air_logger = logging.getLogger("airtest")
air_logger.setLevel(logging.ERROR)


class Logger:
    """
    通过单例模式，只实例化一个日志对象
    直接调用log实例对象进行日志调用
    """

    def __new__(cls, *args, **kwargs):

        if not hasattr(cls, '_instance'):
            cls._instance = super(Logger, cls).__new__(cls)
        return cls._instance

    def __init__(self):
        """
        添加日志记录器，写入文件
        日志分类型存储，一份完整日志，一份错误日志
        通过读取配置文件中的 IS_WRITE 判断是否需要写入文件，便于调试
        """

        self.logger = loguru.logger
        date = time.strftime("%Y-%m-%d %H-%M-%S", time.localtime(time.time()))
        self.adb_log = None

        # if OUTPUT:
        #     all_log_path = os.path.join(REPORT_DIR, "all.log")
        #     self.logger.add(
        #         all_log_path,  # 日志存放位置
        #         retention=7,  # 清理周期
        #         level="DEBUG",  # 日志级别
        #         enqueue=True,  # 具有使日志记录调用非阻塞的优点
        #         encoding="utf-8",
        #         format=LOG_FORMAT
        #     )
        #     error_log_path = os.path.join(REPORT_DIR, "error.log")
        #     self.logger.add(
        #         error_log_path,
        #         retention=7,
        #         level="ERROR",
        #         enqueue=True,
        #         encoding="utf-8",
        #         format=LOG_FORMAT
        #     )

    def get_logger(self):
        return self.logger

    def random_emoji(self):
        emoji = '🍇🍈🍉🍊🍋🍌🍍🥭🍎🍏🍑🍒🍓🥝🍅🥥🥑🍆🥔🥕🌽🥒🥬🥦🧄🧅🍄🎃🎄🎆🎇🧨✨🎈🎉🎊🎋🎍🎎🎏🍖🍗🥩🥓🍔🍟🍕'
        return random.choice(emoji)

    def step(self, msg):
        self.logger.info(f'{" ".join(self.random_emoji() * 2)} ：{msg}')

    def info(self, msg):
        self.logger.info(msg)

    def debug(self, msg):
        self.logger.debug(msg)

    def warn(self, msg):
        self.logger.warning(f'{" ".join(self.random_emoji() * 2)} ：{msg}')

    def error(self, msg):
        self.logger.error(f'😈 😈 ：{msg}')

    def test(self, msg):
        self.logger.info(f'{" ".join(self.random_emoji() * 2)} ：{msg}')
        air.log(f'{" ".join(self.random_emoji() * 2)} ：{msg}')

    def wrap(self, msg: str = None) -> Callable:
        """
        函数日志装饰器
        :param msg: 消息内容
        :return:
        """

        def wrapper(func):
            @functools.wraps(func)
            def inner(*args, **kwargs):
                start_time = time.time()
                try:
                    res = func(*args, **kwargs)
                    end_time = time.time()
                    duration = round(end_time - start_time, 2)
                    self.step(f"{msg} - func: {func.__name__} - duration: {duration} s")
                    if self.adb_log is not None and "Unity" in self.adb_log:
                        log_error_plus(self.adb_log,
                                       desc=f":😈 😈 {msg} - func: {func.__name__}",
                                       snapshot=True)
                    elif tag_listener.tag == -1:
                        log_error_plus("疑似资源缺失，检查上面步骤中的画面",
                                       desc=f":😈 😈 {msg} - func: {func.__name__}",
                                       snapshot=True)
                    else:
                        air.log(f"{' '.join(self.random_emoji() * 2)}: {msg} - func: {func.__name__}", snapshot=True)
                    self.adb_log = None
                    return res
                except Exception as e:
                    air.log(e, desc=f"😈 😈  : {msg} - func: {func.__name__}", snapshot=True)
                    return func

            return inner

        return wrapper

    def log_adb_out(self, adb_log):
        self.adb_log = adb_log

    def tag(self, msg: str = None) -> Callable:
        """
        函数日志装饰器
        :param msg: 消息内容
        :return:
        """

        def tag_wrapper(func):
            @functools.wraps(func)
            def inner(*args, **kwargs):
                start_time = time.time()
                res = func(*args, **kwargs)
                end_time = time.time()
                duration = round(end_time - start_time, 2)
                self.step(f"{msg} - func: {func.__name__} - duration: {duration} s")
                return res

            return inner

        return tag_wrapper

    def case(self, func: Callable[[str], str]) -> Callable:
        """
        测试用例日志装饰器
        :param func: 用例函数对象
        :return:
        """

        @functools.wraps(func)
        def inner(*args, **kwargs):
            self.step(f"Start running testcase: {func.__name__}")
            start_time = time.time()
            res = func(*args, **kwargs)
            end_time = time.time()
            duration = round(end_time - start_time, 2)
            self.step(f"End running testcase : {func.__name__} [ Case duration: {duration} s ]")
            self.step(f"{'- ' * 16} 分割线 {' -' * 16}")
            return res

        return inner


log = Logger()

if __name__ == '__main__':
    log.info('aaaa')
    log.error('aaaa')
    log.warn('aaaa')
    log.step('aaaa')


    @log.case
    def test():
        log.debug(111)


    test()



# !/usr/bin/env python
# -*- coding: utf-8 -*-

"""模板匹配.

对用户提供的调节参数:
    1. threshod: 筛选阈值，默认为0.8
    2. rgb: 彩色三通道,进行彩色权识别.
"""

import cv2
import time

from airtestProject.airtest.utils.logger import get_logger
from .utils import generate_result, check_source_larger_than_search, img_mat_rgb_2_gray, print_run_time
from .cal_confidence import cal_rgb_confidence

LOGGING = get_logger(__name__)


class TemplateMatching(object):
    """模板匹配."""

    METHOD_NAME = "Template"
    MAX_RESULT_COUNT = 10

    def __init__(self, im_search, im_source, threshold=0.8, rgb=True):
        super(TemplateMatching, self).__init__()
        self.im_source = im_source
        self.im_search = im_search
        self.threshold = threshold
        self.rgb = rgb

    @print_run_time
    def find_all_results(self):
        """基于模板匹配查找多个目标区域的方法."""
        # 第一步：校验图像输入
        check_source_larger_than_search(self.im_source, self.im_search)

        # 第二步：计算模板匹配的结果矩阵res
        res = self._get_template_result_matrix()

        # 第三步：依次获取匹配结果
        result = []
        h, w = self.im_search.shape[:2]

        while True:
            # 本次循环中,取出当前结果矩阵中的最优值
            min_val, max_val, min_loc, max_loc = cv2.minMaxLoc(res)
            # 求取可信度:
            confidence = self._get_confidence_from_matrix(max_loc, max_val, w, h)

            if confidence < self.threshold or len(result) > self.MAX_RESULT_COUNT:
                break

            # 求取识别位置: 目标中心 + 目标区域:
            middle_point, rectangle = self._get_target_rectangle(max_loc, w, h)
            one_good_match = generate_result(middle_point, rectangle, confidence)

            result.append(one_good_match)

            # 屏蔽已经取出的最优结果,进入下轮循环继续寻找:
            # cv2.floodFill(res, None, max_loc, (-1000,), max(max_val, 0), flags=cv2.FLOODFILL_FIXED_RANGE)
            cv2.rectangle(res, (int(max_loc[0] - w / 2), int(max_loc[1] - h / 2)), (int(max_loc[0] + w / 2), int(max_loc[1] + h / 2)), (0, 0, 0), -1)

        return result if result else None

    @print_run_time
    def find_best_result(self):
        """基于kaze进行图像识别，只筛选出最优区域."""
        """函数功能：找到最优结果."""
        # 第一步：校验图像输入
        check_source_larger_than_search(self.im_source, self.im_search)
        # 第二步：计算模板匹配的结果矩阵res
        res = self._get_template_result_matrix()
        # 第三步：依次获取匹配结果
        min_val, max_val, min_loc, max_loc = cv2.minMaxLoc(res)
        h, w = self.im_search.shape[:2]
        # 求取可信度:
        confidence = self._get_confidence_from_matrix(max_loc, max_val, w, h)
        # 求取识别位置: 目标中心 + 目标区域:
        middle_point, rectangle = self._get_target_rectangle(max_loc, w, h)
        best_match = generate_result(middle_point, rectangle, confidence)
        LOGGING.debug("[%s] threshold=%s, result=%s" % (self.METHOD_NAME, self.threshold, best_match))

        return best_match if confidence >= self.threshold else None

    def _get_confidence_from_matrix(self, max_loc, max_val, w, h):
        """根据结果矩阵求出confidence."""
        # 求取可信度:
        if self.rgb:
            # 如果有颜色校验,对目标区域进行BGR三通道校验:
            img_crop = self.im_source[max_loc[1]:max_loc[1] + h, max_loc[0]: max_loc[0] + w]
            confidence = cal_rgb_confidence(img_crop, self.im_search)
        else:
            confidence = max_val

        return confidence

    def _get_template_result_matrix(self):
        """求取模板匹配的结果矩阵."""
        # 灰度识别: cv2.matchTemplate( )只能处理灰度图片参数
        s_gray, i_gray = img_mat_rgb_2_gray(self.im_search), img_mat_rgb_2_gray(self.im_source)
        return cv2.matchTemplate(i_gray, s_gray, cv2.TM_CCOEFF_NORMED)

    def _get_target_rectangle(self, left_top_pos, w, h):
        """根据左上角点和宽高求出目标区域."""
        x_min, y_min = left_top_pos
        # 中心位置的坐标:
        x_middle, y_middle = int(x_min + w / 2), int(y_min + h / 2)
        # 左下(min,max)->右下(max,max)->右上(max,min)
        left_bottom_pos, right_bottom_pos = (x_min, y_min + h), (x_min + w, y_min + h)
        right_top_pos = (x_min + w, y_min)
        # 点击位置:
        middle_point = (x_middle, y_middle)
        # 识别目标区域: 点序:左上->左下->右下->右上, 左上(min,min)右下(max,max)
        rectangle = (left_top_pos, left_bottom_pos, right_bottom_pos, right_top_pos)

        return middle_point, rectangle



# This file is part of Androguard.
#
# Copyright (C) 2010, Anthony Desnos <desnos at t0t0.fr>
# All rights reserved.
#
# Androguard is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Androguard is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with Androguard.  If not, see <http://www.gnu.org/licenses/>.


from . import bytecode
from . import typeconstants as tc
from .bytecode import SV
from .stringblock import StringBlock


class AXMLParser:

    def __init__(self, raw_buff):
        self.reset()

        self.buff = bytecode.BuffHandle(raw_buff)

        self.buff.read(4)
        self.buff.read(4)

        self.sb = StringBlock(self.buff)

        self.m_resourceIDs = []
        self.m_prefixuri = {}
        self.m_uriprefix = {}
        self.m_prefixuriL = []

    def reset(self):
        self.m_event = -1
        self.m_lineNumber = -1
        self.m_name = -1
        self.m_namespaceUri = -1
        self.m_attributes = []
        self.m_idAttribute = -1
        self.m_classAttribute = -1
        self.m_styleAttribute = -1

    def next(self):
        self.doNext()
        return self.m_event

    def doNext(self):
        if self.m_event == tc.END_DOCUMENT:
            return

        event = self.m_event

        self.reset()

        while 1:
            chunkType = -1

            # Fake END_DOCUMENT event.
            if event == tc.END_TAG:
                pass

            # START_DOCUMENT
            if event == tc.START_DOCUMENT:
                chunkType = tc.CHUNK_XML_START_TAG
            else:
                if self.buff.end() == True:
                    self.m_event = tc.END_DOCUMENT
                    break
                chunkType = SV('<L', self.buff.read(4)).get_value()

            if chunkType == tc.CHUNK_RESOURCEIDS:
                chunkSize = SV('<L', self.buff.read(4)).get_value()
                # FIXME
                if chunkSize < 8 or chunkSize%4 != 0:
                    raise("ooo")

                for i in range(0, int(chunkSize/4-2)):
                    self.m_resourceIDs.append(SV('<L', self.buff.read(4)))

                continue

            # FIXME
            if chunkType < tc.CHUNK_XML_FIRST or chunkType > tc.CHUNK_XML_LAST:
                raise("ooo")

            # Fake START_DOCUMENT event.
            if chunkType == tc.CHUNK_XML_START_TAG and event == -1:
                self.m_event = tc.START_DOCUMENT
                break

            self.buff.read(4) #/*chunkSize*/
            lineNumber = SV('<L', self.buff.read(4)).get_value()
            self.buff.read(4) #0xFFFFFFFF

            if chunkType == tc.CHUNK_XML_START_NAMESPACE or chunkType == tc.CHUNK_XML_END_NAMESPACE:
                if chunkType == tc.CHUNK_XML_START_NAMESPACE:
                    prefix = SV('<L', self.buff.read(4)).get_value()
                    uri = SV('<L', self.buff.read(4)).get_value()

                    self.m_prefixuri[ prefix ] = uri
                    self.m_uriprefix[ uri ] = prefix
                    self.m_prefixuriL.append((prefix, uri))
                else:
                    self.buff.read(4)
                    self.buff.read(4)
                    (prefix, uri) = self.m_prefixuriL.pop()
                    #del self.m_prefixuri[ prefix ]
                    #del self.m_uriprefix[ uri ]

                continue

            self.m_lineNumber = lineNumber

            if chunkType == tc.CHUNK_XML_START_TAG:
                self.m_namespaceUri = SV('<L', self.buff.read(4)).get_value()
                self.m_name = SV('<L', self.buff.read(4)).get_value()

                # FIXME
                self.buff.read(4) #flags

                attributeCount = SV('<L', self.buff.read(4)).get_value()
                self.m_idAttribute = (attributeCount>>16) - 1
                attributeCount = attributeCount & 0xFFFF
                self.m_classAttribute = SV('<L', self.buff.read(4)).get_value()
                self.m_styleAttribute = (self.m_classAttribute>>16) - 1

                self.m_classAttribute = (self.m_classAttribute & 0xFFFF) - 1

                for i in range(0, attributeCount * tc.ATTRIBUTE_LENGTH):
                    self.m_attributes.append(SV('<L', self.buff.read(4)).get_value())

                for i in range(tc.ATTRIBUTE_IX_VALUE_TYPE, len(self.m_attributes), tc.ATTRIBUTE_LENGTH):
                    self.m_attributes[i] = (self.m_attributes[i]>>24)

                self.m_event = tc.START_TAG
                break

            if chunkType == tc.CHUNK_XML_END_TAG:
                self.m_namespaceUri = SV('<L', self.buff.read(4)).get_value()
                self.m_name = SV('<L', self.buff.read(4)).get_value()
                self.m_event = tc.END_TAG
                break

            if chunkType == tc.CHUNK_XML_TEXT:
                self.m_name = SV('<L', self.buff.read(4)).get_value()

                # FIXME
                self.buff.read(4) #?
                self.buff.read(4) #?

                self.m_event = tc.TEXT
                break

    def getPrefixByUri(self, uri):
        try:
            return self.m_uriprefix[ uri ]
        except KeyError:
            return -1

    def getPrefix(self):
        try:
            return self.sb.getRaw(self.m_prefixuri[ self.m_namespaceUri ])
        except KeyError:
            return ""

    def getName(self):
        if self.m_name == -1 or (self.m_event != tc.START_TAG and self.m_event != tc.END_TAG):
            return ""

        return self.sb.getRaw(self.m_name)

    def getText(self):
        if self.m_name == -1 or self.m_event != tc.TEXT:
            return ""

        return self.sb.getRaw(self.m_name)

    def getNamespacePrefix(self, pos):
        prefix = self.m_prefixuriL[ pos ][0]
        return self.sb.getRaw(prefix)

    def getNamespaceUri(self, pos):
        uri = self.m_prefixuriL[ pos ][1]
        return self.sb.getRaw(uri)

    def getNamespaceCount(self, pos):
        pass

    def getAttributeOffset(self, index):
        # FIXME
        if self.m_event != tc.START_TAG:
            raise("Current event is not START_TAG.")

        offset = index * 5
        # FIXME
        if offset >= len(self.m_attributes):
            raise("Invalid attribute index")

        return offset

    def getAttributeCount(self):
        if self.m_event != tc.START_TAG:
            return -1

        return int(len(self.m_attributes) / tc.ATTRIBUTE_LENGTH)

    def getAttributePrefix(self, index):
        offset = self.getAttributeOffset(index)
        uri = self.m_attributes[offset + tc.ATTRIBUTE_IX_NAMESPACE_URI]

        prefix = self.getPrefixByUri(uri)
        if prefix == -1:
            return ""

        return self.sb.getRaw(prefix)

    def getAttributeName(self, index):
        offset = self.getAttributeOffset(index)
        name = self.m_attributes[offset + tc.ATTRIBUTE_IX_NAME]

        if name == -1:
            return ""

        return self.sb.getRaw(name)

    def getAttributeValueType(self, index):
        offset = self.getAttributeOffset(index)
        return self.m_attributes[offset + tc.ATTRIBUTE_IX_VALUE_TYPE]

    def getAttributeValueData(self, index):
        offset = self.getAttributeOffset(index)
        return self.m_attributes[offset + tc.ATTRIBUTE_IX_VALUE_DATA]

    def getAttributeValue(self, index):
        offset = self.getAttributeOffset(index)
        valueType = self.m_attributes[offset + tc.ATTRIBUTE_IX_VALUE_TYPE]
        if valueType == tc.TYPE_STRING:
            valueString = self.m_attributes[offset + tc.ATTRIBUTE_IX_VALUE_STRING]
            return self.sb.getRaw(valueString)
        # WIP
        return ""
        #int valueData=m_attributes[offset+ATTRIBUTE_IX_VALUE_DATA];
        #return TypedValue.coerceToString(valueType,valueData);




# coding=utf-8

import uiautomation as UIAuto
from airtestProject.poco.sdk.exceptions import UnableToSetAttributeException
from airtestProject.poco.sdk.AbstractNode import AbstractNode


class WindowsUINode(AbstractNode):

    def __init__(self, control, dumper):
        self.Control = control
        self.Children = []
        self.dumper = dumper

    def getParent(self):
        return WindowsUINode(self.Control.GetParentControl(), self.dumper)

    def getChildren(self):
        if len(self.Children):  # 防止多次获取children，提高性能
            for node in self.Children:
                yield WindowsUINode(node, self.dumper)
        else:
            self.Children = self.Control.GetChildren()
            for node in self.Children:
                yield WindowsUINode(node, self.dumper)

    def getAttr(self, attrName):

        if attrName == 'name':
            strr = self.Control.Name
            if strr != "":
                return strr
            return self.Control.ControlTypeName.replace("Control", "")
            
        if attrName == 'originType':
            return self.Control.ControlTypeName

        if attrName == 'type':
            return self.Control.ControlTypeName.replace("Control", "")

        if attrName == 'pos':
            rect = self.Control.BoundingRectangle
            # 计算比例
            pos1 = float(rect[0] + (rect[2] - rect[0]) / 2.0 - self.dumper.RootLeft) / float(self.dumper.RootWidth)
            pos2 = float(rect[1] + (rect[3] - rect[1]) / 2.0 - self.dumper.RootTop) / float(self.dumper.RootHeight)
            return [pos1, pos2]

        if attrName == 'size':
            rect = self.Control.BoundingRectangle
            pos1 = float(rect[2] - rect[0]) / float(self.dumper.RootWidth)
            pos2 = float(rect[3] - rect[1]) / float(self.dumper.RootHeight)
            return [pos1, pos2]

        if attrName == 'text':
            # 先判断控件是否有text属性
            if ((isinstance(self.Control, UIAuto.ValuePattern) and self.Control.IsValuePatternAvailable())):
                return self.Control.CurrentValue()
            else:
                return None

        if attrName == '_instanceId':
            return self.Control.Handle
            
        return super(WindowsUINode, self).getAttr(attrName)

    def setAttr(self, attrName, val):
        if attrName != 'text':
            raise UnableToSetAttributeException(attrName, self)
        else:
            if ((isinstance(self.Control, UIAuto.ValuePattern) and self.Control.IsValuePatternAvailable())):
                self.Control.SetValue(val)
                return True
            else:
                raise UnableToSetAttributeException(attrName, self)

    def getAvailableAttributeNames(self):
        if ((isinstance(self.Control, UIAuto.ValuePattern) and self.Control.IsValuePatternAvailable())):
            return super(WindowsUINode, self).getAvailableAttributeNames() + ('text', '_instanceId', 'originType')
        else:
            return super(WindowsUINode, self).getAvailableAttributeNames() + ('_instanceId', 'originType')



# -*- coding: utf-8 -*-
import os
import time
import threading
import traceback
from airtestProject.airtest.utils.snippet import reg_cleanup, is_exiting, on_method_ready, kill_proc
from airtestProject.airtest.utils.nbsp import NonBlockingStreamReader
from airtestProject.airtest.utils.logger import get_logger
from airtestProject.airtest.core.android.constant import ORI_METHOD, ROTATIONWATCHER_JAR
LOGGING = get_logger(__name__)


class RotationWatcher(object):
    """
    RotationWatcher class
    """

    def __init__(self, adb, ori_method=ORI_METHOD.MINICAP):
        self.adb = adb
        self.ow_proc = None
        self.nbsp = None
        self.ow_callback = []
        self.ori_method = ori_method
        self._t = None
        self._t_kill_event = threading.Event()
        self.current_orientation = None
        self.path_in_android = "/data/local/tmp/" + os.path.basename(ROTATIONWATCHER_JAR)
        reg_cleanup(self.teardown)

    @on_method_ready('start')
    def get_ready(self):
        pass

    def install(self):
        """
        Install the RotationWatcher package

        Returns:
            None

        """
        try:
            exists_file = self.adb.file_size(self.path_in_android)
        except:
            pass
        else:
            local_minitouch_size = int(os.path.getsize(ROTATIONWATCHER_JAR))
            if exists_file and exists_file == local_minitouch_size:
                LOGGING.debug("install_rotationwatcher skipped")
                return
            self.uninstall()

        self.adb.push(ROTATIONWATCHER_JAR, self.path_in_android)
        self.adb.shell("chmod 755 %s" % self.path_in_android)
        LOGGING.info("install rotationwacher finished")

    def uninstall(self):
        """
        Uninstall the RotationWatcher package

        Returns:
            None

        """
        self.adb.raw_shell("rm %s" % self.path_in_android)

    def setup_server(self):
        """
        Setup rotation wacher server

        Returns:
            server process

        """
        self.install()
        if self.ow_proc:
            self.ow_proc.kill()
            self.ow_proc = None

        p = self.adb.start_shell(
            "app_process -Djava.class.path={0} /data/local/tmp com.example.rotationwatcher.Main".format(
                self.path_in_android))
        self.nbsp = NonBlockingStreamReader(p.stdout, name="rotation_server", auto_kill=True)

        if p.poll() is not None:
            # server setup error, may be already setup by others
            # subprocess exit immediately
            raise RuntimeError("rotation watcher server quit immediately")
        self.ow_proc = p
        return p

    def teardown(self):
        self._t_kill_event.set()
        if self.ow_proc:
            kill_proc(self.ow_proc)
        if self.nbsp:
            self.nbsp.kill()
        self.ow_callback = []
        setattr(self, "_start_ready", None)

    def start(self):
        """
        Start the RotationWatcher daemon thread

        Returns:
            initial orientation

        """
        if self.ori_method == ORI_METHOD.MINICAP:
            try:
                self.setup_server()
            except:
                # install or setup failed
                LOGGING.error(traceback.format_exc())
                LOGGING.error("RotationWatcher setup failed, use ADBORI instead.")
                self.ori_method = ORI_METHOD.ADB

        def _refresh_by_ow():
            # 在产生旋转时，nbsp读取到的内容为b"90\r\n"，平时读到的是空数据None，进程结束时读到的是b""
            line = self.nbsp.readline()
            if line is not None:
                if line == b"":
                    self.teardown()
                    if LOGGING is not None:  # may be None atexit
                        LOGGING.debug("orientationWatcher has ended")
                    else:
                        print("orientationWatcher has ended")
                    return None

                ori = int(int(line) / 90)
                return ori
            # 每隔1秒读取一次
            time.sleep(1)

        def _refresh_by_adb():
            ori = self.adb.getDisplayOrientation()
            return ori

        def _run(kill_event):
            while not kill_event.is_set():
                if self.ori_method == ORI_METHOD.ADB:
                    ori = _refresh_by_adb()
                    if self.current_orientation == ori:
                        time.sleep(3)
                        continue
                else:
                    ori = _refresh_by_ow()
                if ori is None:
                    # 以前ori=None是进程结束，现在屏幕方向不变时会返回None
                    continue
                LOGGING.info('update orientation %s->%s' % (self.current_orientation, ori))
                self.current_orientation = ori
                if is_exiting():
                    self.teardown()
                for cb in self.ow_callback:
                    try:
                        cb(ori)
                    except:
                        LOGGING.error("cb: %s error" % cb)
                        traceback.print_exc()

        self.current_orientation = _refresh_by_ow() if self.ori_method != ORI_METHOD.ADB else _refresh_by_adb()

        self._t = threading.Thread(target=_run, args=(self._t_kill_event, ), name="rotationwatcher")
        self._t.daemon = True
        self._t.start()

        return self.current_orientation

    def reg_callback(self, ow_callback):
        """

        Args:
            ow_callback:

        Returns:

        """
        """方向变化的时候的回调函数，参数一定是ori，如果断掉了，ori传None"""
        if ow_callback not in self.ow_callback:
            self.ow_callback.append(ow_callback)


class XYTransformer(object):
    """
    transform the coordinates (x, y) by orientation (upright <--> original)
    """
    @staticmethod
    def up_2_ori(tuple_xy, tuple_wh, orientation):
        """
        Transform the coordinates upright --> original

        Args:
            tuple_xy: coordinates (x, y)
            tuple_wh: screen width and height
            orientation: orientation

        Returns:
            transformed coordinates (x, y)

        """
        x, y = tuple_xy
        w, h = tuple_wh

        if orientation == 1:
            x, y = w - y, x
        elif orientation == 2:
            x, y = w - x, h - y
        elif orientation == 3:
            x, y = y, h - x
        return x, y

    @staticmethod
    def ori_2_up(tuple_xy, tuple_wh, orientation):
        """
        Transform the coordinates original --> upright

        Args:
            tuple_xy: coordinates (x, y)
            tuple_wh: screen width and height
            orientation: orientation

        Returns:
            transformed coordinates (x, y)

        """
        x, y = tuple_xy
        w, h = tuple_wh

        if orientation == 1:
            x, y = y, w - x
        elif orientation == 2:
            x, y = w - x, h - y
        elif orientation == 3:
            x, y = h - y, x
        return x, y



# -*- coding: UTF-8 -*-
# https://stackoverflow.com/questions/14489013/simulate-python-keypresses-for-controlling-a-game

import ctypes
from ctypes.wintypes import DWORD, LONG, WORD, WPARAM
import win32con


# Extended keys and their corresponding scancodes
EXTENDED_KEYS = {
    'NUMPAD_ENTER': 0x9C,
    'RCTRL': 0x9D,
    'NUMPAD_/': 0xB5,
    'RALT': 0xB8,
    'HOME': 0xC7,
    'UP': 0xC8,
    'PAGE_UP': 0xC9,
    'LEFT': 0xCB,
    'RIGHT': 0xCD,
    'END': 0xCF,
    'DOWN': 0xD0,
    'PAGE_DOWN': 0xD1,
    'INSERT': 0xD2,
    'DELETE': 0xD3,
    'LWINDOWS': 0xDB,
    'RWINDOWS': 0xDC,
    'MENU': 0xDD
}

# Keys and their corresponding scancodes
KEYS = {
    'ESCAPE': 0x01,
    '1': 0x02,
    '2': 0x03,
    '3': 0x04,
    '4': 0x05,
    '5': 0x06,
    '6': 0x07,
    '7': 0x08,
    '8': 0x09,
    '9': 0x0A,
    '0': 0x0B,
    '-': 0x0C,
    '=': 0x0D,
    'BACKSPACE': 0x0E,
    'TAB': 0x0F,
    'Q': 0x10,
    'W': 0x11,
    'E': 0x12,
    'R': 0x13,
    'T': 0x14,
    'Y': 0x15,
    'U': 0x16,
    'I': 0x17,
    'O': 0x18,
    'P': 0x19,
    '[': 0x1A,
    ']': 0x1B,
    'ENTER': 0x1C,
    'LCTRL': 0x1D,
    'A': 0x1E,
    'S': 0x1F,
    'D': 0x20,
    'F': 0x21,
    'G': 0x22,
    'H': 0x23,
    'J': 0x24,
    'K': 0x25,
    'L': 0x26,
    ';': 0x27,
    "'": 0x28,
    '`': 0x29,
    'LSHIFT': 0x2A,
    'BACKSLASH': 0x2B,
    'Z': 0x2C,
    'X': 0x2D,
    'C': 0x2E,
    'V': 0x2F,
    'B': 0x30,
    'N': 0x31,
    'M': 0x32,
    ',': 0x33,
    '.': 0x34,
    '/': 0x35,
    'RSHIFT': 0x36,
    '*': 0x37,
    'LALT': 0x38,
    'SPACE': 0x39,
    'CAPS_LOCK': 0x3A,
    'F1': 0x3B,
    'F2': 0x3C,
    'F3': 0x3D,
    'F4': 0x3E,
    'F5': 0x3F,
    'F6': 0x40,
    'F7': 0x41,
    'F8': 0x42,
    'F9': 0x43,
    'F10': 0x44,
    'NUM_LOCK': 0x45,
    'SCROLL_LOCK': 0x46,
    'NUMPAD_7': 0x47,
    'NUMPAD_8': 0x48,
    'NUMPAD_9': 0x49,
    'NUMPAD_-': 0x4A,
    'NUMPAD_4': 0x4B,
    'NUMPAD_5': 0x4C,
    'NUMPAD_6': 0x4D,
    'NUMPAD_+': 0x4E,
    'NUMPAD_1': 0x4F,
    'NUMPAD_2': 0x50,
    'NUMPAD_3': 0x51,
    'NUMPAD_0': 0x52,
    'NUMPAD_.': 0x53,
    'F11': 0x57,
    'F12': 0x58,
    'PRINT_SCREEN': 0xB7,
    'PAUSE': 0xC5
}

KEYEVENTF_SCANCODE = 0x0008

# WPARAM is conditionally defined based on the pointer size.
ULONG_PTR = WPARAM


# https://docs.microsoft.com/zh-cn/windows/win32/api/winuser/ns-winuser-keybdinput
class KeyBdInput(ctypes.Structure):
    """
    Contains information about a simulated keyboard event.
    """
    _fields_ = [('wVk', WORD),
                ('wScan', WORD),
                ('dwFlags', DWORD),
                ('time', DWORD),
                ('dwExtraInfo', ULONG_PTR)]


# https://docs.microsoft.com/zh-cn/windows/win32/api/winuser/ns-winuser-mouseinput
class MouseInput(ctypes.Structure):
    """
    Contains information about a simulated mouse event.
    """
    _fields_ = [('dx', LONG),
                ('dy', LONG),
                ('mouseData', DWORD),
                ('dwFlags', DWORD),
                ('time', DWORD),
                ('dwExtraInfo', ULONG_PTR)]


# https://docs.microsoft.com/zh-cn/windows/win32/api/winuser/ns-winuser-hardwareinput
class HardwareInput(ctypes.Structure):
    """
    Contains information about a simulated message generated by an input device
    other than a keyboard or mouse.
    """
    _fields_ = [('uMsg', DWORD),
                ('wParamL', WORD),
                ('wParamH', WORD)]


class _Input(ctypes.Union):
    _fields_ = [('ki', KeyBdInput),
                ('mi', MouseInput),
                ('hi', HardwareInput)]


# https://docs.microsoft.com/zh-cn/windows/win32/api/winuser/ns-winuser-input
class Input(ctypes.Structure):
    """
    Used by SendInput to store information for synthesizing input events such
    as keystrokes, mouse movement, and mouse clicks.
    """
    _anonymous_ = ('_input',)
    _fields_ = [('type', DWORD),
                ('_input', _Input)]


def key_press(key):
    """Simulates a key press event.

    Sends a scancode to the computer to report which key has been pressed.
    Some games use DirectInput devices, and respond only to scancodes, not
    virtual key codes. You can simulate DirectInput key presses using this
    method, instead of the keyevent(...) method, which uses virtual key
    codes.

    :param key: A string indicating which key to be pressed.
                Available key options are listed in KEYS and EXTENDED_KEYS.
    """
    try:
        key_name = key.upper()
    except AttributeError:
        raise ValueError('invalid literal for key_press(): {}'.format(key))
    try:
        hex_code = KEYS[key_name]
    except KeyError:
        pass
    else:
        flags = KEYEVENTF_SCANCODE
        send_keyboard_input(hex_code, flags)
        return
    try:
        hex_code = EXTENDED_KEYS[key_name]
    except KeyError:
        raise ValueError('invalid literal for key_press(): {}'.format(key))
    else:
        flags = win32con.KEYEVENTF_EXTENDEDKEY | KEYEVENTF_SCANCODE
        send_keyboard_input(hex_code, flags)


def key_release(key):
    """Simulates a key release event.

    Sends a scancode to the computer to report which key has been released.
    Some games use DirectInput devices, and respond only to scancodes, not
    virtual key codes. You can simulate DirectInput key releases using this
    method. A call to the key_release(...) method usually follows a call to
    the key_press(..) method of the same key.

    :param key: A string indicating which key to be released.
    """
    try:
        key_name = key.upper()
    except AttributeError:
        raise ValueError('invalid literal for key_release(): {}'.format(key))
    try:
        hex_code = KEYS[key_name]
    except KeyError:
        pass
    else:
        flags = KEYEVENTF_SCANCODE | win32con.KEYEVENTF_KEYUP
        send_keyboard_input(hex_code, flags)
        return
    try:
        hex_code = EXTENDED_KEYS[key_name]
    except KeyError:
        raise ValueError('invalid literal for key_release(): {}'.format(key))
    else:
        flags = (win32con.KEYEVENTF_EXTENDEDKEY | KEYEVENTF_SCANCODE |
                 win32con.KEYEVENTF_KEYUP)
        send_keyboard_input(hex_code, flags)


def send_keyboard_input(hex_code, flags):
    """Simulates a key press/release event with SendInput.

    :param hex_code: Scancode of the particular key to be pressed/released in
                     hexadecimal.
    :param flags: Flags indicating various aspects of the keystroke.
    """
    inputs = Input(type=ctypes.c_ulong(win32con.INPUT_KEYBOARD),
                   ki=KeyBdInput(wScan=hex_code, dwFlags=flags))
    ctypes.windll.user32.SendInput(1, ctypes.byref(inputs),
                                   ctypes.sizeof(inputs))



# -*- coding: utf-8 -*-
import os
import airtestProject.airtest.report.report as report
LOGDIR = "log"
poco_func = ["record_ui"]

old_trans_desc = report.LogToHtml._translate_desc
old_analyse = report.LogToHtml._analyse
old_translate_title = report.LogToHtml._translate_title


def new_analyse(self):

    for log in self.log:
        if log["data"]["name"] == "record_ui":
            log["depth"] = 2

    trans_steps = old_analyse(self)
    return trans_steps


def new_translate_desc(self, step, code):
    trans = old_trans_desc(self, step, code)

    poco_ui = ""
    if "__children__" in step:
        for item in step["__children__"]:
            if item["data"]["name"] == "record_ui":
                poco_ui = item["data"]["call_args"]["ui"]

    if poco_ui:
        name = step['data']['name']
        desc = {
            "touch": "Touch %s" % poco_ui,
            "swipe": "Swipe from %s" % poco_ui,
            "set_text": "Set %s of text" % poco_ui
        }

        ret = desc.get(name)
        if callable(ret):
            ret = ret()
        return ret

    else:
        return trans


def new_translate_title(self, name, step):
    title = old_translate_title(self, name, step)

    poco_title = False
    if "__children__" in step:
        for item in step["__children__"]:
            if item["data"]["name"] == "record_ui":
                poco_title = True

    if poco_title:
        title = {
            "touch": u"Poco Click",
            "swipe": u"Poco Swipe",
            "set_text": u"Poco Set Text"
        }
        return title.get(name, name)
    else:
        return title


report.LogToHtml._translate_desc = new_translate_desc
report.LogToHtml._analyse = new_analyse
report.LogToHtml._translate_title = new_translate_title



# coding=utf-8
from airtestProject.poco.freezeui.hierarchy import FrozenUIDumper
from airtestProject.poco.utils.simplerpc.utils import sync_wrapper


class StdDumper(FrozenUIDumper):
    def __init__(self, rpcclient):
        super(StdDumper, self).__init__()
        self.rpcclient = rpcclient

    @sync_wrapper
    def dumpHierarchy(self, onlyVisibleNode=True):
        return self.rpcclient.call("Dump", onlyVisibleNode)



# coding=utf-8


class Transport(object):
    def update(self, timeout=None):
        raise NotImplementedError

    def send(self, cid, data):
        raise NotImplementedError

    def recv(self):
        raise NotImplementedError

    def connect(self, endpoint):
        raise NotImplementedError

    def disconnect(self, endpoint=None):
        raise NotImplementedError

    def bind(self, endpoint):
        raise NotImplementedError



"""
基于poco对象、air二次封装的操作方法
"""
from airtestProject.commons.utils.exception import NoSuchNodeException
from airtestProject.poco.exceptions import PocoNoSuchNodeException, PocoTargetRemovedException
from airtestProject.abstractBase.OperateBase import OperateABC

from airtestProject.poco.drivers.unity3d import UnityPoco
import re
import time
from airtestProject.airtest.core import api as air
from airtestProject.commons.utils.logger import log
import time

identifier: str = '-'
index_compile = re.compile(r'(?<=\[)\d+?(?=\\])')
input_type = "InputField"


class Page:

    def __init__(self, poco_instance):

        self.poco = poco_instance or UnityPoco()

    def get_poco(self, pos):
        return self.parser_pos(pos)

    def click(self, pos, focus=None, sleep_interval: float = None):
        try:
            return self.parser_pos(pos).click(focus, sleep_interval)
        except PocoNoSuchNodeException:
            try:
                log.debug(f"尝试再次点击： {pos}")
                return self.parser_pos(pos).click(focus, sleep_interval)
            except PocoNoSuchNodeException:
                raise NoSuchNodeException(f'找不到节点 "{pos}"')

    def exists(self, pos):
        try:
            return self.parser_pos(pos).exists()
        except (PocoTargetRemovedException, PocoNoSuchNodeException):
            return False

    def long_click(self, pos: str, duration: float = 2.0):
        return self.parser_pos(pos).long_click(duration)

    # def double_click(self, pos: str, focus=None, sleep_interval: float = None):
    #     return self.parser_pos(pos).double_click(focus, sleep_interval)
    #
    # def rclick(self, pos: str, focus=None, sleep_interval: float = None):
    #     return self.parser_pos(pos).rclick(focus, sleep_interval)

    def swipe(self, pos: str, direction, focus=None, duration: float = 0.5):
        return self.parser_pos(pos).swipe(direction, focus, duration)

    def scroll(self, pos: str, direction='vertical', percent: float = 0.6, duration: float = 2.0):
        return self.parser_pos(pos).scroll(direction, percent, duration)

    def pinch(self, pos: str, direction='in', percent: float = 0.6, duration: float = 2.0, dead_zone: float = 0.1):
        return self.parser_pos(pos).pinch(direction, percent, duration, dead_zone)

    def set_text(self, pos: str, text: str):
        pos = self.parser_pos(pos)
        if pos.attr("type") == input_type:
            return pos.set_text(text)
        else:
            pos.click()
            for i in range(20):
                self.keyevent("KEYCODE_DEL")
            self.text(text)

    def get_text(self, pos: str):
        return self.parser_pos(pos).get_text()

    def get_name(self, pos: str):
        return self.parser_pos(pos).get_name()

    def drag_to(self, pos: str, target: str, duration=0.5):
        return self.parser_pos(pos).drag_to(self.parser_pos(target), duration)

    def wait_for_any(self, pos_list: list, timeout=30):
        return self.poco.wait_for_any(self.parser_pos_list(pos_list), timeout)

    def wait_for_all(self, pos_list: list, timeout=30):
        return self.poco.wait_for_all(self.parser_pos_list(pos_list), timeout)

    def wait_for_appearance(self, pos, timeout=30):
        return self.parser_pos(pos).wait_for_appearance(timeout)

    def wait_for_disappearance(self, pos, timeout=30):
        return self.parser_pos(pos).wait_for_disappearance(timeout)

    def sleep(self, secs: float = 1.0):
        log.step(f"sleep {secs} seconds .")
        time.sleep(secs)

    def regex_pos_index(self, pos: str):
        """
        正则匹配定位的下标
        :param pos:
        :return:
        """
        s = index_compile.search(pos)
        if s:
            index = s.group()
            rep_pos = pos.replace(f"[{index}]", "")
            return rep_pos, int(index)
        return pos, 0

    def parser_pos(self, pos: str):
        """
        解析pos定位，基于poco定位方式，支持更简便的一行式书写定位方法，支持五种定位方法；
        传入的pos为中文时，使用text文本定位，不以中文开头不会匹配text文本；
        属性定位与正则定位属于同一种书写方式，均采用关键字方式书写，例如：text=确定、textMatches=确定；
        传入的pos为元素时，使用普通方式定位，并且当存在多级定位时，使用 - 符号连结，程序自动拆解，并自动解析index；
        相对定位与顺序定位可结合使用，采用先切割后取值的方式解析定位，例如：Bg_Front[1]-Close；
        :param pos: 定位
        :return:
        example：
            基本定位："Btn_Enter"
            顺序定位："Bg_Front[1]"
            相对定位: "Bg_Front[1]-Close"
            属性定位："text=确定"
            正则定位："textMatches=确定"
        """

        if '=' in pos:
            attr, pos = pos.split('=')
            return self.poco(**{attr: pos})

        if identifier not in pos:
            if index_compile.search(pos):
                rep_pos, index = self.regex_pos_index(pos)
                return self.poco(rep_pos)[index]

            return self.poco(pos)

        value_list = pos.split(identifier)
        pos_list = [self.regex_pos_index(value) for value in value_list]

        p0, n0 = pos_list[0]
        p1, n1 = pos_list[1]

        return self.poco(p0)[n0].child(p1)[n1]

    def parser_pos_list(self, pos_list: list):
        poco_list = []

        for pos in pos_list:
            poco_list.append(self.poco(pos))

        return poco_list

    def shell(self, cmd):
        try:
            log.info(f"启动远程shell并执行命令成功: {cmd}")
            return self.air.shell(cmd)
        except Exception as e:
            log.error(f"启动远程shell并执行命令失败: {cmd}")
            raise e

    def start_app(self, package, activity=None):
        try:
            self.air.start_app(package, activity)
            log.info(f"启动APP: {package}, activity: {activity}")
        except Exception as e:
            log.error(f"启动APP失败 {package}, activity: {activity}")
            raise e

    def stop_app(self, package):
        try:
            self.air.stop_app(package)
            log.info(f"停止APP, package: {package}")
        except Exception as e:
            log.error(f"停止APP失败, package: {package}")
            raise e

    def clear_app(self, package):
        try:
            self.air.clear_app(package)
            log.info(f"清除APP的数据, package: {package}")
        except Exception as e:
            log.error(f"清除APP的数据失败, package: {package}")
            raise e

    def snapshot(self, filename=None, msg="", quality=None, max_size=None):
        try:
            log.info(f"设备执行截图:{filename}.")
            return self.air.snapshot(filename, msg, quality, max_size)
        except Exception as e:
            log.error(f"设备执行截图失败")
            raise e

    def wake(self):
        try:
            log.info(f"设备执行滑动解锁")
            return self.air.wake
        except Exception as e:
            log.error(f"设备执行滑动解锁失败")
            raise e

    def home(self, package):
        raise NotImplementedError

    def touch(self, v, times=1, **kwargs):
        try:
            log.info(f"设备上执行点击, pos: {v}")
            return self.air.touch(v, times, **kwargs)
        except Exception as e:
            log.error(f"设备上点击失败, pos: {v}")
            raise e

    def double_click(self, v1, v2=None, vector=None, **kwargs):
        log.info(f"设备上执行双击")
        return self.air.double_click(v1, v2, vector, **kwargs)

    # def swipe(self,v1, v2=None, vector=None, **kwargs):
    #     log.info("设备上执行滑动")
    #     return self.air.swipe(v1, v2, vector, **kwargs)

    # def pinch(self,in_or_out='in', center=None, percent=0.5):
    #     log.info(f"设备上执行缩放")
    #     return self.air.pinch(in_or_out, center, percent)

    def keyevent(self, keyname, **kwargs):
        log.info(f"设备上执行: {keyname} 按键")
        return self.air.keyevent(keyname, **kwargs)

    def text(self, value, enter=True, **kwargs):
        try:
            log.info(f"输入 {value} ")
            return self.air.text(value, enter, **kwargs)
        except Exception as e:
            log.error(f"输入 {value} 失败")
            raise e

    def wait(self):
        raise NotImplementedError

    # def exists(self,v):
    #     try:
    #         log.info(f"节点: {v} 存在.")
    #         return self.air.exists(v)
    #     except Exception as e:
    #         log.error(f"节点: {v} 不存在.")
    #         raise e




# coding=utf-8

import math
from airtestProject.poco.utils.track import MotionTrack


def make_pinching(direction, center, size, percent, dead_zone, duration):
    w, h = size
    half_distance = percent / 2
    dead_zone_distance = dead_zone / 2
    pa0 = center
    pb0 = list(pa0)
    pa1 = list(pa0)
    pb1 = list(pa0)
    if direction == 'in':
        pa0[0] += w * half_distance
        pa0[1] += h * half_distance
        pb0[0] -= w * half_distance
        pb0[1] -= h * half_distance
        pa1[0] += w * dead_zone_distance
        pa1[1] += h * dead_zone_distance
        pb1[0] -= w * dead_zone_distance
        pb1[1] -= h * dead_zone_distance
    else:
        pa1[0] += w * half_distance
        pa1[1] += h * half_distance
        pb1[0] -= w * half_distance
        pb1[1] -= h * half_distance
        pa0[0] += w * dead_zone_distance
        pa0[1] += h * dead_zone_distance
        pb0[0] -= w * dead_zone_distance
        pb0[1] -= h * dead_zone_distance

    speed = math.sqrt(w * h) * (percent - dead_zone) / 2 / duration
    track_a = MotionTrack([pa0, pa1], speed)
    track_b = MotionTrack([pb0, pb1], speed)

    return track_a, track_b


def make_panning():
    pass



import os
import requests

from airtestProject import config
from airtest.utils.logger import get_logger

from airtestProject.manager.LogManager import LogManager, catch_error

"""
author liuhuatong
des 包管理模块
date 2024/4/29
"""


log_manager = LogManager()
class PackageManager:
    @catch_error
    @staticmethod
    def download_file(link, project=None):
        """根据链接下载包体
        :param link: 链接
        :return:
        """
        path = os.path.join(config.ROOT_PATH, "packages", project)
        if not os.path.exists(path):
            os.makedirs(path)
        file_path = os.path.join(path, link.split("/")[-1])
        # 发起GET请求，设置stream=True开启流式传输
        with requests.get(link, stream=True) as response:
            if response.status_code == 200:
                # 打开一个文件进行二进制写操作
                with open(file_path, 'wb') as file:
                    # 指定chunk_size大小，逐块读取和保存文件
                    for chunk in response.iter_content(chunk_size=1024*8):
                        # 过滤掉keep-alive的新块
                        if chunk:
                            file.write(chunk)
                log_manager.log_step("{} 文件下载成功!".format(link))
            else:
                log_manager.log_step("{} 文件下载失败，状态码： {}".format(link, response.status_code))

    @catch_error
    def install_file(self, device_id, file_path, platform=None):
        """安装包体
        :param device_id: 设备id
        :param file_path: 包路径
        :param platform: 平台
        :return: Bool()
        """
        if not platform:
            result = os.popen('adb -s %s install -r %s' % (device_id, file_path)).read()
            log_manager.log_step('install_file {} {}'.format(device_id, result))
            if "success" in result.lower():
                return True
            else:
                return False
        else:
            result = os.popen('tidevice --udid %s install %s' % (device_id, file_path)).read()
            pass

    @catch_error
    def uninstall_file(self, device_id, package_name, platform=None):
        """卸载包体
        :param device_id: 设备id
        :param package_name: 包名字
        :param platform: 平台
        :return: Bool()
        """
        if not platform:
            result = os.popen('adb -s %s uninstall %s' % (device_id, package_name)).read()
            log_manager.log_step('uninstall_file {} {}'.format(device_id, result))
            if "success" in result.lower():
                return True
            else:
                return False
        else:
            result = os.popen('tidevice --udid %s uninstall %s' % (device_id, package_name)).read()
            pass



# -*- coding: utf-8 -*-

import threading
from airtestProject.poco.drivers.std import StdPoco
from airtestProject.poco.utils.device import VirtualDevice
from airtestProject.poco.drivers.std import DEFAULT_ADDR, DEFAULT_PORT
from airtestProject.poco.utils.simplerpc.utils import sync_wrapper
from airtestProject.poco.exceptions import InvalidOperationException


class WindowsPoco(StdPoco):
    """
    Poco WindowsUI implementation.

    Args:
        selector (:py:obj:`dict`): find window by a selector, optional parameters: ``title``, ``handle``, ``title_re``
         title: find windows by title

         handle: find windows by handle

         title_re: find windows by regular expression of title

        addr (:py:obj:`tuple`): where the WindowsUI running on, (localhost,15004) by default
        options: see :py:class:`poco.pocofw.Poco`

    Examples:
        If your programme is running, you could initialize poco instance by using following snippet::
            from poco.drivers.windows import WindowsPoco
            # poco = WindowsPoco({'title':'xxx'})
            # poco = WindowsPoco({'handle':123456})
            # poco = WindowsPoco({'title_re':'[a-z][a-z][a-z]'})

    """   
    
    def __init__(self, selector=None, addr=DEFAULT_ADDR, **options):
        if 'action_interval' not in options:
            options['action_interval'] = 0.1

        if addr[0] == "localhost" or addr[0] == "127.0.0.1":
            from poco.drivers.windows.sdk.WindowsUI import PocoSDKWindows
            self.sdk = PocoSDKWindows(addr)
            self.SDKProcess = threading.Thread(target=self.sdk.run)  # 创建线程
            self.SDKProcess.setDaemon(True)
            self.SDKProcess.start()

        dev = VirtualDevice(addr[0])
        super(WindowsPoco, self).__init__(addr[1], dev, False, **options)
        
        self.selector = selector
        self.connect_window(self.selector)

    @sync_wrapper
    def connect_window(self, selector):
        return self.agent.rpc.call("ConnectWindow", selector)

    @sync_wrapper
    def set_foreground(self):
        return self.agent.rpc.call("SetForeground")

    def scroll(self, direction='vertical', percent=1, duration=2.0):
        # 重写Win下的Scroll函数，percent代表滑动滚轮多少次，正数为向上滑，负数为向下滑，direction无用，只能上下滚
        if direction not in ('vertical', 'horizontal'):
            raise ValueError('Argument `direction` should be one of "vertical" or "horizontal". Got {}'.format(repr(direction)))
        if direction is 'horizontal':
            raise InvalidOperationException("Windows does not support horizontal scrolling currently")
            
        return self.agent.input.scroll(direction, percent, duration)

    def rclick(self, pos):
        return self.agent.input.rclick(pos[0], pos[1])

    def double_click(self, pos):
        return self.agent.input.double_click(pos[0], pos[1])

    def keyevent(self, keyname):
        return self.agent.input.keyevent(keyname)



"""
随机数据生成类
"""
from faker import Faker
fake = Faker()

def random_phone_number() -> str:
    """随机手机号"""
    return fake.phone_number()


def random_country() -> str:
    """随机国家"""
    return fake.country()


def random_city() -> str:
    """随机城市"""
    return fake.city()


def random_city_suffix() -> str:
    """随机县"""
    return fake.city_suffix()


def random_address() -> str:
    """随机地址"""
    return fake.address()


def random_street_address() -> str:
    """街道"""
    return fake.street_address()


def random_street_name() -> str:
    """街道名"""
    return fake.street_name()


def random_name() -> str:
    """随机姓名"""
    return fake.name()


def random_name_female() -> str:
    """女性姓名"""
    return fake.name_female()


def random_ean8():
    """8位条码"""
    return fake.ean8()


def random_ean13():
    """13位条码"""
    return fake.ean13()


def random_ean():
    """自定义位数条码,只能选8或者13"""
    return fake.ean(length=8)


def random_company() -> str:
    """公司名"""
    return fake.company()


def random_credit_card_number():
    """卡号"""
    return fake.credit_card_number(card_type=None)


def random_credit_card_full():
    """完整卡信息"""
    return fake.credit_card_full(card_type=None)


def random_date_time():
    """随机日期时间"""
    return fake.date_time(tzinfo=None)


def random_date_time_between():
    """两个时间间的一个随机时间"""
    return fake.date_time_between(start_date="-30y", end_date="now", tzinfo=None)


def random_day_of_month():
    """随机月中某一天"""
    return fake.day_of_month()


def random_unix_time() -> int:
    """随机unix时间（时间戳"""
    return fake.unix_time()


def random_date() -> str:
    """随机日期（可自定义格式"""
    return fake.date(pattern="%Y-%m-%d")


def random_job() -> str:
    """工作职位"""
    return fake.job()


def random_text() -> str:
    """随机生成一篇文章"""
    return fake.text(max_nb_chars=200)


def random_word() -> str:
    """随机单词"""
    return fake.word()


def random_sentence() -> str:
    """随机生成一个句子"""
    return fake.sentence(nb_words=6, variable_nb_words=True)


def random_paragraph() -> str:
    """随机生成一段文字(字符串)"""
    return fake.paragraph(nb_sentences=3, variable_nb_sentences=True)


def random_password() -> str:
    """随机密码（可指定密码策略)"""
    return fake.password(length=10, special_chars=True, digits=True, upper_case=True, lower_case=True)


def random_uuid4() -> str:
    """随机uuid"""
    return fake.uuid4()


def random_pyint() -> int:
    """随机int"""
    return fake.pyint()


def random_pystr() -> str:
    """随机字符串（可指定长度）"""
    return fake.pystr(min_chars=None, max_chars=20)


def random_num(length=1) -> int:
    """随机数字"""
    return fake.random_number(length)


def random_email() -> str:
    """随机email"""
    return fake.email()


def random_ipv4() -> str:
    """随机IPV4地址"""
    return fake.ipv4()


if __name__ == '__main__':
    v = random_name()
    print(v)
    print(type(v))



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
Detect keypoints with BRIEF/SIFT/SURF.
Need opencv-contrib module.
"""

import cv2

from .error import *  # noqa
from .keypoint_base import KeypointMatching


def check_cv_version_is_new():
    """opencv版本是3.0或4.0以上, API接口与2.0的不同."""
    if cv2.__version__.startswith("3.") or cv2.__version__.startswith("4."):
        return True
    else:
        return False


class BRIEFMatching(KeypointMatching):
    """FastFeature Matching."""

    METHOD_NAME = "BRIEF"  # 日志中的方法名

    def init_detector(self):
        """Init keypoint detector object."""
        # BRIEF is a feature descriptor, recommand CenSurE as a fast detector:
        if check_cv_version_is_new():
            # OpenCV3/4, star/brief is in contrib module, you need to compile it seperately.
            try:
                self.star_detector = cv2.xfeatures2d.StarDetector_create()
                self.brief_extractor = cv2.xfeatures2d.BriefDescriptorExtractor_create()
            except:
                import traceback
                traceback.print_exc()
                print("to use %s, you should build contrib with opencv3.0" % self.METHOD_NAME)
                raise NoModuleError("There is no %s module in your OpenCV environment !" % self.METHOD_NAME)
        else:
            # OpenCV2.x
            self.star_detector = cv2.FeatureDetector_create("STAR")
            self.brief_extractor = cv2.DescriptorExtractor_create("BRIEF")

        # create BFMatcher object:
        self.matcher = cv2.BFMatcher(cv2.NORM_L1)  # cv2.NORM_L1 cv2.NORM_L2 cv2.NORM_HAMMING(not useable)

    def get_keypoints_and_descriptors(self, image):
        """获取图像特征点和描述符."""
        # find the keypoints with STAR
        kp = self.star_detector.detect(image, None)
        # compute the descriptors with BRIEF
        keypoints, descriptors = self.brief_extractor.compute(image, kp)
        return keypoints, descriptors

    def match_keypoints(self, des_sch, des_src):
        """Match descriptors (特征值匹配)."""
        # 匹配两个图片中的特征点集，k=2表示每个特征点取出2个最匹配的对应点:
        return self.matcher.knnMatch(des_sch, des_src, k=2)


class SIFTMatching(KeypointMatching):
    """SIFT Matching."""

    METHOD_NAME = "SIFT"  # 日志中的方法名

    # SIFT识别特征点匹配，参数设置:
    FLANN_INDEX_KDTREE = 0

    def init_detector(self):
        """Init keypoint detector object."""
        if check_cv_version_is_new():
            try:
                # opencv3 >= 3.4.12 or opencv4 >=4.5.0, sift is in main repository
                self.detector = cv2.SIFT_create(edgeThreshold=10)
            except AttributeError:
                try:
                    self.detector = cv2.xfeatures2d.SIFT_create(edgeThreshold=10)
                except:
                    raise NoModuleError(
                        "There is no %s module in your OpenCV environment, need contrib module!" % self.METHOD_NAME)
        else:
            # OpenCV2.x
            self.detector = cv2.SIFT(edgeThreshold=10)

        # # create FlnnMatcher object:
        self.matcher = cv2.FlannBasedMatcher({'algorithm': self.FLANN_INDEX_KDTREE, 'trees': 5}, dict(checks=50))

    def get_keypoints_and_descriptors(self, image):
        """获取图像特征点和描述符."""
        keypoints, descriptors = self.detector.detectAndCompute(image, None)
        return keypoints, descriptors

    def match_keypoints(self, des_sch, des_src):
        """Match descriptors (特征值匹配)."""
        # 匹配两个图片中的特征点集，k=2表示每个特征点取出2个最匹配的对应点:
        return self.matcher.knnMatch(des_sch, des_src, k=2)


class SURFMatching(KeypointMatching):
    """SURF Matching."""

    METHOD_NAME = "SURF"  # 日志中的方法名

    # 是否检测方向不变性:0检测/1不检测
    UPRIGHT = 0
    # SURF算子的Hessian Threshold
    HESSIAN_THRESHOLD = 400
    # SURF识别特征点匹配方法设置:
    FLANN_INDEX_KDTREE = 0

    def init_detector(self):
        """Init keypoint detector object."""
        # BRIEF is a feature descriptor, recommand CenSurE as a fast detector:
        if check_cv_version_is_new():
            # OpenCV3/4, surf is in contrib module, you need to compile it seperately.
            try:
                self.detector = cv2.xfeatures2d.SURF_create(self.HESSIAN_THRESHOLD, upright=self.UPRIGHT)
            except:
                raise NoModuleError("There is no %s module in your OpenCV environment, need contribmodule!" % self.METHOD_NAME)
        else:
            # OpenCV2.x
            self.detector = cv2.SURF(self.HESSIAN_THRESHOLD, upright=self.UPRIGHT)

        # # create FlnnMatcher object:
        self.matcher = cv2.FlannBasedMatcher({'algorithm': self.FLANN_INDEX_KDTREE, 'trees': 5}, dict(checks=50))

    def get_keypoints_and_descriptors(self, image):
        """获取图像特征点和描述符."""
        keypoints, descriptors = self.detector.detectAndCompute(image, None)
        return keypoints, descriptors

    def match_keypoints(self, des_sch, des_src):
        """Match descriptors (特征值匹配)."""
        # 匹配两个图片中的特征点集，k=2表示每个特征点取出2个最匹配的对应点:
        return self.matcher.knnMatch(des_sch, des_src, k=2)



#!/usr/bin/env python
# -*- coding:utf8 -*-
import json
import os
import io
import re
import six
import sys
from PIL import Image
import shutil
import jinja2
import traceback
from copy import deepcopy
from datetime import datetime
from markupsafe import Markup, escape
from six.moves.urllib.parse import parse_qsl, urlparse

try:
    from jinja2 import evalcontextfilter as pass_eval_context  # jinja2<3.1
except:
    from jinja2 import pass_eval_context  # jinja2>=3.1
from airtestProject.airtest.aircv import imread, get_resolution
from airtestProject.airtest.aircv.error import FileNotExistError
from airtestProject.airtest.core.settings import Settings as ST
from airtestProject.airtest.aircv.utils import compress_image
from airtestProject.airtest.utils.compat import decode_path, script_dir_name
from airtestProject.airtest.cli.info import get_script_info
from airtestProject.airtest.utils.logger import get_logger
from airtestProject.airtest.utils.snippet import parse_device_uri
from six import PY3

LOGGING = get_logger(__name__)
# 将log地址改为airtestProjects/log目录下
DEFAULT_LOG_DIR = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
DEFAULT_LOG_FILE = "log.txt"
# HTML_TPL = "log_template.html"
# HTML_TPL = "D:\\Users\Desktop\odin-testautomation\TestAutomation\\airtestProject\\reports\log_template.html"
HTML_TPL = "log_template.html"
HTML_FILE = "log.html"
STATIC_DIR = os.path.join(os.path.dirname(os.path.dirname(__file__)), 'reports')

_paragraph_re = re.compile(r'(?:\r\n|\r|\n){2,}')


def nl2br(eval_ctx, value):
    result = u'\n\n'.join(u'<p>%s</p>' % p.replace('\n', '<br>\n')
                          for p in _paragraph_re.split(escape(value)))
    if eval_ctx.autoescape:
        result = Markup(result)
    return result


def timefmt(timestamp):
    """
    Formatting of timestamp in Jinja2 templates
    :param timestamp: timestamp of steps
    :return: "%Y-%m-%d %H:%M:%S"
    """
    return datetime.fromtimestamp(timestamp).strftime("%Y-%m-%d %H:%M:%S")


class ReportManager(object):
    """Convert log to html display """
    scale = 0.5

    def __init__(self, script_root, log_root=None, static_root="", export_dir=None, script_name="", logfile=None,
                 lang="en", plugins=['poco.utils.airtest.reports'], project=None):
        if plugins is None:
            plugins = ["poco.utils.airtest.reports"]
        self.log = []
        self.devices = {}
        self.script_root = script_root
        self.script_name = script_name
        # self.project = project
        if not self.script_name or os.path.isfile(self.script_root):
            self.script_root, self.script_name = script_dir_name(self.script_root)
        project_dir = os.path.join(DEFAULT_LOG_DIR, 'log', project)
        if not os.path.exists(project_dir):
            os.makedirs(project_dir)
        self.log_root = log_root or project_dir
        self.static_root = static_root or STATIC_DIR
        self.test_result = True
        self.run_start = None
        self.run_end = None
        self.export_dir = export_dir
        log_file = os.path.join(self.log_root, DEFAULT_LOG_FILE)
        if not os.path.exists(log_file):
            with open(log_file, "w") as f:
                pass
        self.logfile = log_file
        self.lang = lang
        self.init_plugin_modules(plugins)

    @staticmethod
    def init_plugin_modules(plugins):
        if not plugins:
            return
        for plugin_name in plugins:
            LOGGING.debug("try loading plugin: %s" % plugin_name)
            try:
                __import__(plugin_name)
            except:
                LOGGING.error(traceback.format_exc())

    def _load(self):
        logfile = os.path.join(self.log_root, self.logfile)
        if not PY3:
            logfile = logfile.encode(sys.getfilesystemencoding())
        with io.open(logfile, encoding="utf-8") as f:
            for line in f.readlines():
                self.log.append(json.loads(line))

    def _analyse(self):
        """ 解析log成可渲染的dict """
        steps = []
        children_steps = []

        for log in self.log:
            depth = log['depth']

            if not self.run_start:
                self.run_start = log.get('data', {}).get('start_time', '') or log["time"]
            self.run_end = log["time"]

            if depth == 0:
                # single log line, not in stack
                steps.append(log)
            elif depth == 1:
                step = deepcopy(log)
                step["__children__"] = children_steps
                steps.append(step)
                children_steps = []
            else:
                children_steps.insert(0, log)

        translated_steps = [self._translate_step(s) for s in steps]
        if len(translated_steps) > 0 and translated_steps[-1].get("traceback"):
            # Final Error
            self.test_result = False
        return translated_steps

    def _translate_step(self, step):
        """translate single step"""
        name = step["data"]["name"]
        title = self._translate_title(name, step)
        code = self._translate_code(step)
        desc = self._translate_desc(step, code)
        screen = self._translate_screen(step, code)
        info = self._translate_info(step)
        assertion = self._translate_assertion(step)
        self._translate_device(step)

        translated = {
            "title": title,
            "time": step["time"],
            "code": code,
            "screen": screen,
            "desc": desc,
            "traceback": info[0],
            "log": info[1],
            "assert": assertion,
        }
        return translated

    @staticmethod
    def _translate_assertion(step):
        if "assert_" in step["data"].get("name", "") and "msg" in step["data"].get("call_args", {}):
            return step["data"]["call_args"]["msg"]

    def _translate_screen(self, step, code):
        if step['tag'] not in ["function", "info"] or not step.get("__children__"):
            return None
        screen = {
            "src": None,
            "rect": [],
            "pos": [],
            "vector": [],
            "confidence": None,
        }

        for item in step["__children__"]:
            if item["data"]["name"] == "try_log_screen":
                snapshot = item["data"].get("ret", None)
                if isinstance(snapshot, six.text_type):
                    src = snapshot
                elif isinstance(snapshot, dict):
                    src = snapshot['screen']
                    screen['resolution'] = snapshot['resolution']
                else:
                    continue
                if self.export_dir:  # all relative path
                    screen['_filepath'] = os.path.join(DEFAULT_LOG_DIR, src)
                else:
                    screen['_filepath'] = os.path.abspath(os.path.join(self.log_root, src))
                screen['src'] = screen['_filepath']
                self.get_thumbnail(os.path.join(self.log_root, src))
                screen['thumbnail'] = self.get_small_name(screen['src'])
                break

        display_pos = None

        for item in step["__children__"]:
            if item["data"]["name"] == "_cv_match" and isinstance(item["data"].get("ret"), dict):
                cv_result = item["data"]["ret"]
                pos = cv_result['result']
                if self.is_pos(pos):
                    display_pos = [round(pos[0]), round(pos[1])]
                rect = self.div_rect(cv_result['rectangle'])
                screen['rect'].append(rect)
                screen['confidence'] = cv_result['confidence']
                break

        if step["data"]["name"] in ["touch", "assert_exists", "wait", "exists"]:
            # 将图像匹配得到的pos修正为最终pos
            if self.is_pos(step["data"].get("ret")):
                display_pos = step["data"]["ret"]
            elif self.is_pos(step["data"]["call_args"].get("v")):
                display_pos = step["data"]["call_args"]["v"]

        elif step["data"]["name"] == "swipe":
            if "ret" in step["data"]:
                screen["pos"].append(step["data"]["ret"][0])
                target_pos = step["data"]["ret"][1]
                origin_pos = step["data"]["ret"][0]
                screen["vector"].append([target_pos[0] - origin_pos[0], target_pos[1] - origin_pos[1]])

        if display_pos:
            screen["pos"].append(display_pos)
        return screen

    def _translate_device(self, step):
        if step["tag"] == "function" and step["data"]["name"] == "connect_device":
            uri = step["data"]["call_args"]["uri"]
            platform, uuid, params = parse_device_uri(uri)
            if "name" in params:
                uuid = params["name"]
            if uuid not in self.devices:
                self.devices[uuid] = uri
            return uuid
        return None

    @classmethod
    def get_thumbnail(cls, path):
        """compress screenshot"""
        new_path = cls.get_small_name(path)
        if not os.path.isfile(new_path):
            try:
                img = Image.open(path)
                compress_image(img, new_path, ST.SNAPSHOT_QUALITY, max_size=300)
            except Exception:
                LOGGING.error(traceback.format_exc())
            return new_path
        else:
            return None

    @classmethod
    def get_small_name(cls, filename):
        print('filename:', filename)
        name, ext = os.path.splitext(filename)
        return "%s_small%s" % (name, ext)

    def _translate_info(self, step):
        trace_msg, log_msg = "", ""
        if "traceback" in step["data"]:
            # 若包含有traceback内容，将会认定步骤失败
            trace_msg = step["data"]["traceback"]
        if step["tag"] == "info":
            if "log" in step["data"]:
                # 普通文本log内容，仅显示
                log_msg = step["data"]["log"]
        return trace_msg, log_msg

    def _translate_code(self, step):
        if step["tag"] != "function":
            return None
        step_data = step["data"]
        args = []
        code = {
            "name": step_data["name"],
            "args": args,
        }
        for key, value in step_data["call_args"].items():
            args.append({
                "key": key,
                "value": value,
            })
        for k, arg in enumerate(args):
            value = arg["value"]
            if isinstance(value, dict) and value.get("__class__") == "Template":
                if self.export_dir:  # all relative path
                    image_path = value['filename']
                    if not os.path.isfile(os.path.join(self.script_root, image_path)) and value['_filepath']:
                        # copy image used by using statement
                        shutil.copyfile(value['_filepath'], os.path.join(self.script_root, value['filename']))
                else:
                    image_path = os.path.abspath(value['_filepath'] or value['filename'])
                arg["image"] = image_path
                try:
                    if not value['_filepath'] and not os.path.exists(value['filename']):
                        crop_img = imread(os.path.join(self.script_root, value['filename']))
                    else:
                        crop_img = imread(value['_filepath'] or value['filename'])
                except FileNotExistError:
                    # 在某些情况下会报图片不存在的错误（偶现），但不应该影响主流程
                    if os.path.exists(image_path):
                        arg["resolution"] = get_resolution(imread(image_path))
                    else:
                        arg["resolution"] = (0, 0)
                else:
                    arg["resolution"] = get_resolution(crop_img)
        return code

    @staticmethod
    def div_rect(r):
        """count rect for js use"""
        xs = [p[0] for p in r]
        ys = [p[1] for p in r]
        left = min(xs)
        top = min(ys)
        w = max(xs) - left
        h = max(ys) - top
        return {'left': left, 'top': top, 'width': w, 'height': h}

    def _translate_desc(self, step, code):
        """ 函数描述 """
        if step['tag'] != "function":
            return None
        name = step['data']['name']
        res = step['data'].get('ret')
        args = {i["key"]: i["value"] for i in code["args"]}

        desc = {
            "snapshot": lambda: u"Screenshot description: %s" % args.get("msg"),
            "touch": lambda: u"Touch %s" % (
                "target image" if isinstance(args['v'], dict) else "coordinates %s" % args['v']),
            "swipe": u"Swipe on screen",
            "wait": u"Wait for target image to appear",
            "exists": lambda: u"Image %s exists" % ("" if res else "not"),
            "text": lambda: u"Input text:%s" % args.get('text'),
            "keyevent": lambda: u"Click [%s] button" % args.get('keyname'),
            "sleep": lambda: u"Wait for %s seconds" % args.get('secs'),
            "assert_exists": u"Assert target image exists",
            "assert_not_exists": u"Assert target image does not exists",
            "connect_device": lambda: u"Connect device: %s" % args.get("uri"),
        }

        # todo: 最好用js里的多语言实现
        desc_zh = {
            "snapshot": lambda: u"截图描述: %s" % args.get("msg"),
            "touch": lambda: u"点击 %s" % (u"目标图片" if isinstance(args['v'], dict) else u"屏幕坐标 %s" % args['v']),
            "swipe": u"滑动操作",
            "wait": u"等待目标图片出现",
            "exists": lambda: u"图片%s存在" % ("" if res else u"不"),
            "text": lambda: u"输入文字:%s" % args.get('text'),
            "keyevent": lambda: u"点击[%s]按键" % args.get('keyname'),
            "sleep": lambda: u"等待%s秒" % args.get('secs'),
            "assert_exists": u"断言目标图片存在",
            "assert_not_exists": u"断言目标图片不存在",
            "connect_device": lambda: u"连接设备： %s" % args.get("uri"),
        }

        if self.lang == "zh":
            desc = desc_zh

        ret = desc.get(name)
        if callable(ret):
            ret = ret()
        return ret

    @staticmethod
    def _translate_title(name, step):
        title = {
            "touch": u"Touch",
            "swipe": u"Swipe",
            "wait": u"Wait",
            "exists": u"Exists",
            "text": u"Text",
            "keyevent": u"Keyevent",
            "sleep": u"Sleep",
            "assert_exists": u"Assert exists",
            "assert_not_exists": u"Assert not exists",
            "snapshot": u"Snapshot",
            "assert_equal": u"Assert equal",
            "assert_not_equal": u"Assert not equal",
            "connect_device": u"Connect device",
        }

        return title.get(name, name)

    @staticmethod
    def _render(template_name, output_file=None, **template_vars):
        """ 用jinja2渲染html"""
        # todo: 这里的STATIC_DIR可以改成report目录路径
        env = jinja2.Environment(
            loader=jinja2.FileSystemLoader(STATIC_DIR),
            extensions=(),
            autoescape=True
        )
        env.filters['nl2br'] = nl2br
        env.filters['datetime'] = timefmt
        template = env.get_template(template_name)
        html = template.render(**template_vars)

        if output_file:
            with io.open(output_file, 'w', encoding="utf-8") as f:
                f.write(html)
            LOGGING.info(output_file)

        return html

    def is_pos(self, v):
        return isinstance(v, (list, tuple))

    def copy_tree(self, src, dst, ignore=None):
        try:
            shutil.copytree(src, dst, ignore=ignore)
        except:
            LOGGING.error(traceback.format_exc())

    def _make_export_dir(self):
        """mkdir & copy /staticfiles/screenshots"""
        # let dirname = <script name>.log
        dirname = self.script_name.replace(os.path.splitext(self.script_name)[1], ".log")
        # mkdir
        dirpath = os.path.join(self.export_dir, dirname)
        if os.path.isdir(dirpath):
            shutil.rmtree(dirpath, ignore_errors=True)

        # copy script
        def ignore_export_dir(dirname, filenames):
            # 忽略当前导出的目录，防止递归导出
            if os.path.commonprefix([dirpath, dirname]) == dirpath:
                return filenames
            return []
        # self.copy_tree(self.script_root, dirpath, ignore=ignore_export_dir)
        # copy log
        logpath = os.path.join(dirpath, DEFAULT_LOG_DIR)
        if os.path.normpath(logpath) != os.path.normpath(self.log_root):
            if os.path.isdir(logpath):
                shutil.rmtree(logpath, ignore_errors=True)
            self.copy_tree(self.log_root, logpath, ignore=shutil.ignore_patterns(dirname))
        # if self.static_root is not a http server address, copy static files from local directory
        if not self.static_root.startswith("http"):
            for subdir in ["css", "fonts", "image", "js"]:
                self.copy_tree(os.path.join(self.static_root, subdir), os.path.join(dirpath, "static", subdir))

        return dirpath, logpath

    def get_relative_log(self, output_file):
        """
        Try to get the relative path of log.txt
        :param output_file: output file: log.html
        :return: ./log.txt or ""
        """
        try:
            html_dir = os.path.dirname(output_file)
            if self.export_dir:
                # When exporting reports, the log directory will be named log/ (DEFAULT_LOG_DIR),
                # so the relative path of log.txt is log/log.txt
                return os.path.join(DEFAULT_LOG_DIR, os.path.basename(self.logfile))
            return os.path.relpath(os.path.join(self.log_root, self.logfile), html_dir)
        except:
            LOGGING.error(traceback.format_exc())
            return ""

    def get_console(self, output_file):
        html_dir = os.path.dirname(output_file)
        file = os.path.join(html_dir, 'console.txt')
        content = ""
        if os.path.isfile(file):
            try:
                content = self.readFile(file)
            except Exception:
                try:
                    content = self.readFile(file, "gbk")
                    print(content)
                except Exception:
                    content = traceback.format_exc() + content
                    content = content + "Can not read console.txt. Please check file in:\n" + file
        return content

    def readFile(self, filename, code='utf-8'):
        content = ""
        with io.open(filename, encoding=code) as f:
            for line in f.readlines():
                content = content + line
        return content

    def report_data(self, output_file=None, record_list=None):
        """
        Generate data for the reports page

        :param output_file: The file name or full path of the output file, default HTML_FILE
        :param record_list: List of screen recording files
        :return:
        """
        self._load()
        steps = self._analyse()

        script_path = os.path.join(self.script_root, self.script_name)
        info = json.loads(get_script_info(script_path))
        info['devices'] = self.devices

        if record_list:
            records = [os.path.join(DEFAULT_LOG_DIR, f) if self.export_dir
                       else os.path.abspath(os.path.join(self.log_root, f)) for f in record_list]
        else:
            records = []

        if not self.static_root.endswith(os.path.sep):
            self.static_root = self.static_root.replace("\\", "/")
            self.static_root += "/"

        if not output_file:
            output_file = HTML_FILE

        data = {}
        data['steps'] = steps
        data['name'] = self.script_root
        data['scale'] = self.scale
        data['test_result'] = self.test_result
        data['run_end'] = self.run_end
        data['run_start'] = self.run_start
        data['static_root'] = self.static_root
        data['lang'] = self.lang
        data['records'] = records
        data['info'] = info
        data['log'] = self.get_relative_log(output_file)
        data['console'] = self.get_console(output_file)
        # 如果带有<>符号，容易被highlight.js认为是特殊语法，有可能导致页面显示异常，尝试替换成不常用的{}
        info = json.dumps(data).replace("<", "{").replace(">", "}")
        data['data'] = info
        return data

    def report(self, template_name=HTML_TPL, output_file=HTML_FILE, record_list=None):
        """
       生成报表页面，您可以添加自定义数据并在需要时重载它

        :param template_name: 模板名字，默认是 HTML_TPL
        :param output_file:输出文件的文件名或完整路径，默认为HTML_file
        :param record_list: 屏幕录制文件列表
        :return:
        """
        if not self.script_name:
            path, self.script_name = script_dir_name(self.script_root)

        if self.export_dir:
            script_root, self.log_root = self._make_export_dir()
            # output_file可传入文件名，或绝对路径
            output_file = output_file if output_file and os.path.isabs(output_file) \
                else os.path.join(script_root, output_file or HTML_FILE)
            if not self.static_root.startswith("http"):
                self.static_root = "static/"
        if not record_list:
            record_list = [f for f in os.listdir(self.log_root) if f.endswith(".mp4")]
        data = self.report_data(output_file=output_file, record_list=record_list)
        return self._render(template_name, output_file, **data)


def simple_report(filepath, logpath=True, logfile=None, output=HTML_FILE, project=None):
    path, name = script_dir_name(filepath)
    if logpath is True:
        logpath = os.path.join(path, getattr(ST, "LOG_DIR", DEFAULT_LOG_DIR))
    rpt = ReportManager(path, logpath, logfile=logfile or getattr(ST, "LOG_FILE", DEFAULT_LOG_FILE),
                        script_name=name,
                        project=project)
    # todo 这里的HTML_TPL、output找不到log_template.html，需要修改
    rpt.report(HTML_TPL, output_file=output)
    # _path = HTML_TPL.replace(
    #     '\\', '/')
    # rpt.reports(_path, output_file=output)


def get_parger(ap):
    ap.add_argument("script", help="script filepath")
    ap.add_argument("--outfile", help="output html filepath, default to be log.html", default=HTML_FILE)
    ap.add_argument("--static_root", help="static files root dir")
    ap.add_argument("--log_root", help="log & screen data root dir, logfile should be log_root/log.txt")
    ap.add_argument("--record", help="custom screen record file path", nargs="+")
    ap.add_argument("--export", help="export a portable reports dir containing all resources")
    ap.add_argument("--lang", help="reports language", default="en")
    ap.add_argument("--plugins", help="load reporter plugins", nargs="+")
    ap.add_argument("--reports", help="placeholder for reports cmd", default=True, nargs="?")
    return ap


def main(args):
    # script filepath
    print("ReportManager.main(args):", args)
    path, name = script_dir_name(args.script)
    record_list = args.record or []
    log_root = decode_path(args.log_root) or decode_path(os.path.join(path, DEFAULT_LOG_DIR))
    static_root = args.static_root or STATIC_DIR
    static_root = decode_path(static_root)
    export = decode_path(args.export) if args.export else None
    lang = args.lang if args.lang in ['zh', 'en'] else 'en'
    plugins = args.plugins

    # gen html reports
    rpt = ReportManager(path, log_root, static_root, export_dir=export, script_name=name, lang=lang, plugins=plugins)
    rpt.report(HTML_TPL, output_file=args.outfile, record_list=record_list)


if __name__ == "__main__":
    import argparse

    ap = argparse.ArgumentParser()
    args = get_parger(ap).parse_args()
    main(args)



# This file is part of Androguard.
#
# Copyright (C) 2010, Anthony Desnos <desnos at t0t0.fr>
# All rights reserved.
#
# Androguard is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Androguard is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with Androguard.  If not, see <http://www.gnu.org/licenses/>.

import six
from .bytecode import SV


class StringBlock:
    """
    axml format translated from:
    http://code.google.com/p/android4me/source/browse/src/android/content/res/AXmlResourceParser.java
    """
    def __init__(self, buff):
        buff.read(4)

        self.chunkSize = SV('<L', buff.read(4))
        self.stringCount = SV('<L', buff.read(4))
        self.styleOffsetCount = SV('<L', buff.read(4))

        # unused value ?
        buff.read(4) # ?

        self.stringsOffset = SV('<L', buff.read(4))
        self.stylesOffset = SV('<L', buff.read(4))

        self.m_stringOffsets = []
        self.m_styleOffsets = []
        self.m_strings = []
        self.m_styles = []

        for i in range(0, self.stringCount.get_value()):
            self.m_stringOffsets.append(SV('<L', buff.read(4)))

        for i in range(0, self.styleOffsetCount.get_value()):
            self.m_stylesOffsets.append(SV('<L', buff.read(4)))

        size = self.chunkSize.get_value() - self.stringsOffset.get_value()
        if self.stylesOffset.get_value() != 0:
            size = self.stylesOffset.get_value() - self.stringsOffset.get_value()

        # FIXME
        if (size % 4) != 0:
            pass

        for i in range(0, int(size / 4)):
            self.m_strings.append(SV('=L', buff.read(4)))

        if self.stylesOffset.get_value() != 0:
            size = self.chunkSize.get_value() - self.stringsOffset.get_value()

            # FIXME
            if (size % 4) != 0:
                pass

            for i in range(0, size / 4):
                self.m_styles.append(SV('=L', buff.read(4)))

    def getRaw(self, idx):
        if idx < 0 or self.m_stringOffsets == [] or idx >= len(self.m_stringOffsets):
            return None

        offset = self.m_stringOffsets[ idx ].get_value()
        length = self.getShort(self.m_strings, offset)

        data = ""

        while length > 0:
            offset += 2
            # get the unicode character as the apk might contain non-ASCII label
            data += six.unichr(self.getShort(self.m_strings, offset))

            # FIXME
            if data[-1] == "&":
                data = data[:-1]

            length -= 1

        return data

    def getShort(self, array, offset):
        value = array[int(offset / 4)].get_value()
        if (int((offset % 4)) / 2) == 0:
            return value & 0xFFFF
        else:
            return value >> 16



# coding=utf-8

import base64
from airtestProject.airtest.core.api import snapshot, device as current_device
from airtestProject.poco.sdk.interfaces.screen import ScreenInterface


class AirtestScreen(ScreenInterface):
    def __init__(self):
        super(AirtestScreen, self).__init__()

    def getPortSize(self):
        disp = current_device().display_info
        if disp['orientation'] in (1, 3):
            return [disp['height'], disp['width']]
        else:
            return [disp['width'], disp['height']]

    def getScreen(self, width):
        savepath = snapshot()
        return base64.b64encode(open(savepath, 'rb').read()), 'png'



# -*- coding: utf-8 -*-
__author__ = 'wjjn3033'

import os
import re
import sys
import six
import json
from io import open
from airtestProject.airtest.utils.compat import EXT


def get_script_info(script_path):
    """extract info from script, like basename, __author__, __title__ and __desc__."""
    script_path = os.path.normpath(script_path)
    script_name = os.path.basename(script_path)
    if script_path.endswith(".py"):
        pyfilepath = script_path
        parent_name = os.path.basename(os.path.dirname(script_path))
        if parent_name.endswith(EXT):
            script_name = parent_name
    else:
        pyfilename = script_name.replace(EXT, ".py")
        pyfilepath = os.path.join(script_path, pyfilename)

    if not os.path.exists(pyfilepath) and six.PY2:
        pyfilepath = pyfilepath.encode(sys.getfilesystemencoding())
    with open(pyfilepath, encoding="utf-8") as pyfile:
        pyfilecontent = pyfile.read()

    author, title, desc = get_author_title_desc(pyfilecontent)

    result_json = {"name": script_name, "path": script_path, "author": author, "title": title, "desc": desc}
    return json.dumps(result_json)


def get_author_title_desc(text):
    """Get author title desc."""
    regex1 = r'__(?P<attr>\w+)__\s*=\s*(?P<val>"[^"]+"|"""[^"]+""")'
    regex2 = r"__(?P<attr>\w+)__\s*=\s*(?P<val>'[^']+'|'''[^']+''')"
    data1 = re.findall(regex1, text)
    data2 = re.findall(regex2, text)
    data1.extend(data2)
    file_info = dict(data1)
    author = strip_str(file_info.get("author", ""))
    title = strip_str(file_info.get("title", ""))
    desc = strip_str(file_info.get("desc", ""))
    desc = process_desc(desc)
    return author, title, desc


def process_desc(desc):
    lines = desc.split('\n')
    lines = [line.strip() for line in lines]
    return '\n'.join(lines)


def strip_str(string):
    """Strip string."""
    return string.strip('"').strip("'").strip()



# coding=utf-8

__author__ = 'lxn3032'


class CommandInterface(object):
    """
    This is one of the main communication interfaces. This interface defines command-level behaviours providing 
    abilities to control remote runtime by sending self-defined command. The provided command can be various type -
    from string type to specific structure of a dict.
    """

    def command(self, cmd, type):
        """
        Emit a command to remote runtime (target device). 

        Args:
            cmd: any json serializable data.
            type (:obj:`str`): a string value indicated the command type (command tag).

        Returns:
            None (recommended).
        """

        pass



#!-*- coding = utf-8 -*-
# @Time : 2024/4/7 2:44
# @Author : 苏嘉浩
# @File : OperateBase.py
# @Software : PyCharm
from abc import ABC, abstractmethod

"""
airtest核心api和poco的二次封装，操作具体基类
"""


class OperateABC(ABC):

    @abstractmethod
    def click(self, value, *args, **kwargs):
        """
        :param value:
        :param args:
        :param kwargs: ocrPlus=True,可以开启二值化
        :return:
        """
        pass

    @abstractmethod
    def exists(self, pos, **kwargs):
        """

        :param pos: 传入的元素
        :param kwargs: 如果是使用ocr可以传入ocrPlus=True,可以开启二值化
        :return:
        """
        pass

    @abstractmethod
    def sleep(self, secs):
        """

        :param secs: 延迟秒数
        :return:
        """
        pass

    @abstractmethod
    def swipe(self, value, v2=None, vector_direction=None, **kwargs):
        """

        :param value: 要滑动的元素
        :param v2: airtest可以选填滑动到的元素（poco其实底层也是有两个元素的，但是封装后现在没有，后续考虑airtest也去掉第二个元素）
        :param vector_direction: （按比例滑动）
        :param kwargs: 定义参数，熟悉poco的可以自己添加
        :return:
        """
        raise NotImplementedError

    @abstractmethod
    def pinch(self, in_or_out, percent=0.5, **kwargs):
        """

        :param in_or_out: 缩放方向in就是里面out就是往外
        :param percent: 屏幕捏动百分比
        :param kwargs: airtest和poco还有一些小参数可以自己定义
        :return:
        """
        pass

    @abstractmethod
    def set_text(self, pos, text, *args, **kwargs):
        pass

    @abstractmethod
    def wait(self, pos, timeout=None, *args, **kwargs):
        pass

    @abstractmethod
    def wait_disappear_element(self, pos, *args, **kwargs):
        """
        等待元素消失
        :param pos: 元素
        :param args:
        :param kwargs:
        :return:
        """
        pass

    @abstractmethod
    def wait_element_appear(self, pos, *args, **kwargs):
        """
        等待元素出现
        :param pos:
        :param args:
        :param kwargs:
        :return:
        """
        pass

    @abstractmethod
    def persistent_element_exists(self, pos):
        """
        暂时未实现，判断元素是否一直存在
        :param pos:
        :return:
        """
        pass

    def set_dict(self, script_root, project):
        """
        图片识别需要设定路径用文件名代替路径
        :param script_root: 脚本当前目录
        :param project: 定位到的具体图片文件夹
        :return:
        """
        pass

    @abstractmethod
    def wait_for_any(self, pos_list: list, timeout=30):
        """
        等待列表中的某一个元素
        :param pos_list: 元素列表
        :param timeout:
        :return:
        """
        pass

    @abstractmethod
    def wait_for_all(self, pos_list: list, timeout=30):
        """

        :param pos_list: 等待列表中的全部元素
        :param timeout:
        :return:
        """
        pass

    @abstractmethod
    def wait_next_element(self, last_click_pos, next_pos):
        """
        等待下一个元素，没有出现就点击上一个
        :param last_click_pos: 上一个需要点击的元素
        :param next_pos: 下一个元素
        :return:
        """
        pass

    @abstractmethod
    def get_text(self, pos):
        """
        获取元素的文本内容，airtest暂未实现
        :param pos: 元素
        :return:
        """
        pass

# class OperateClass:
#     def __init__(self, operate: OperateABC):
#         self.operate = operate



# -*- coding: utf-8 -*-
from airtest.cli.parser import get_parser


def main(argv=None):
    ap = get_parser()
    args = ap.parse_args(argv)
    if args.action == "info":
        from airtest.cli.info import get_script_info
        print(get_script_info(args.script))
    elif args.action == "report":
        from airtest.report.report import main as report_main
        report_main(args)
    elif args.action == "run":
        from airtest.cli.runner import run_script
        run_script(args)
    elif args.action == "version":
        from airtest.utils.version import show_version
        show_version()
    else:
        ap.print_help()


if __name__ == '__main__':
    main()



"""
airtest核心api的二次封装，操作具体实现类
"""
import os

from airtestProject.airtest.core import api as air
from airtestProject.airtest.core.error import TargetNotFoundError

from airtestProject.commons.utils.logger import log
from airtestProject.abstractBase.OperateBase import OperateABC
import time

from airtestProject.commons.utils.myTemplate import myTemplate
from airtestProject.commons.utils.ocrTemplate import OcrTemplate
from airtestProject.commons.utils.tools import get_folder_path_up


class myAirTest(OperateABC):

    def __init__(self, language):
        super(myAirTest, self).__init__()
        self.this_dict = None
        self.language = language

    def set_dict(self, script_root: str, project: str):
        """
        各种兼容还有路径转换。ps images文件夹只能在脚本的上级或者同级否则无法正常工作
        :param script_root: 脚本当前目录
        :param project: 定位到的具体图片文件夹
        :return:
        """
        if self.this_dict is None:
            files_dict = {}
            images_abs_path = get_folder_path_up(script_root, 'images')  # 获取图片文件夹的绝对路径
            # script_root_dir = os.path.dirname(script_root)  # 获取脚本文件夹的绝对路径
            # images_relative_path = os.path.relpath(images_abs_path, script_root_dir)  # 构造为图片文件夹的相对路径
            images_project_relative_path = os.path.join(images_abs_path, project)  # 构造为图片文件夹和project的相对路径
            # 遍历指定目录下的所有文件
            for filename in os.listdir(images_project_relative_path):
                file_path = os.path.join(images_project_relative_path, filename)
                if os.path.isfile(file_path):
                    file_name_without_ext, _ = os.path.splitext(filename)
                    files_dict[file_name_without_ext] = myTemplate(file_path)
            print(files_dict)
            self.this_dict = files_dict

    def init_device(self, platform=None, uuid=None, **kwargs):
        try:
            air.init_device(platform=platform, uuid=uuid, **kwargs)
            log.info(f"init device success, platform: {platform}, uuid:  {uuid}")
        except Exception as e:
            log.error(f"failed to init device, platform: {platform}, uuid:  {uuid}")
            raise e

    def connect_device(self, uri):
        log.info('🍇 🍉 ready to connect device host ...')
        try:
            air.connect_device(uri)
            log.info(f'🍊 🍋 connect device succeeded... host: {uri}')
        except ConnectionError as e:
            log.error(f'connect device failed... host: {uri}')
            raise ConnectionError from e

    def device(self):
        current_active_device = air.device()
        log.info(f'get current active device: {current_active_device}')
        return current_active_device

    def set_current(self, idx):
        try:
            log.info(f"set current active device idx: {idx}")
            air.set_current(idx)
        except Exception as e:
            log.error(f"failed to set current active device idx: {idx}")
            raise e

    def auto_setup(self, basedir=None, devices=None, logdir=None, project_root=None, compress=None):
        air.auto_setup(basedir=basedir, devices=devices, logdir=logdir, project_root=project_root, compress=compress)

    def shell(self, cmd):
        try:
            log.info(f"execute adb shell command: {cmd}")
            return air.shell(cmd)
        except Exception as e:
            log.error(f"failed to execute adb shell command: {cmd}")
            raise e

    def start_app(self, package, activity=None):
        try:
            air.start_app(package, activity)
            log.info(f"start app: {package}, activity: {activity}")
        except Exception as e:
            log.error(f"failed start app {package}, activity: {activity}")
            raise e

    def stop_app(self, package):
        try:
            air.stop_app(package)
            log.info(f"stop the application on device, package: {package}")
        except Exception as e:
            log.error(f"failed to stop the application on device, package: {package}")
            raise e

    def clear_app(self, package):
        try:
            air.clear_app(package)
            log.info(f"clear data of the application on device, package: {package}")
        except Exception as e:
            log.error(f"failed to clear data of the application on device, package: {package}")
            raise e

    def install(self, filepath, **kwargs):
        try:
            log.info(f"install application: {filepath}")
            return air.install(filepath, **kwargs)
        except Exception as e:
            log.error(f"failed to install application: {filepath}")
            raise e

    def uninstall(self, package):
        try:
            log.info(f"uninstall application: {package}")
            return air.uninstall(package)
        except Exception as e:
            log.error(f"failed to uninstall application: {package}")
            raise e

    def snapshot(self, filename=None, msg="", quality=None, max_size=None):
        try:
            log.info(f"take the screenshot of the device and save it to the filepath:{filename}.")
            return air.snapshot(filename, msg, quality, max_size)
        except Exception as e:
            log.error(f"failed to wake up and unlock the device")
            raise e

    def wake(self):
        try:
            log.info(f"wake up and unlock the device")
            return air.wake
        except Exception as e:
            log.error(f"failed to wake up and unlock the device")
            raise e

    def home(self, package):
        raise NotImplementedError

    def click(self, value, times=1, ocrPlus=False, **kwargs):
        try:
            log.info(f"perform the touch action on the device screen, pos: {value}")
            if 'focus' in kwargs.keys():
                del kwargs['focus']
            value = self.check_value(value, ocrPlus)
            return air.touch(value, times, **kwargs)
        except Exception as e:
            log.error(f"failed to perform the touch action on the device screen, pos: {value}")
            raise e

    def double_click(self, v1, v2=None, vector=None, **kwargs):
        log.info(f"perform double click")
        return air.double_click(v1, v2, vector, **kwargs)

    def swipe(self, value, v2=None, vector_direction=None, **kwargs):
        if vector_direction is None:
            vector_direction = [0, -3.0]
        log.info("Perform the swipe action on the device screen.")
        value = self.check_value(value)
        if v2 is not None:
            v2 = self.check_value(v2)
        return air.swipe(value, v2, vector=vector_direction, **kwargs)

    def pinch(self, in_or_out='in', percent=0.5, center=None):
        log.info(f"perform the pinch action on the device screen")
        return air.pinch(in_or_out, center, percent)

    def keyevent(self, keyname, **kwargs):
        log.info(f"perform key event keyname: {keyname} on the device.")
        return air.keyevent(keyname, **kwargs)

    def set_text(self, pos=None, text=None, enter=True, **kwargs):
        try:
            self.click(pos)
            log.info(f"input {text} on the target device")
            for i in range(20):
                air.keyevent("KEYCODE_DEL")
                log.info(f"我在删除")
            return air.text(text, enter, **kwargs)
        except Exception as e:
            log.error(f"failed to input {text} on the target device")
            raise e

    def sleep(self, secs=1.0):
        time.sleep(secs)
        log.info(f'time sleep {secs} seconds')

    def wait(self, pos, timeout=None, interval=0.5, intervalfunc=None):
        try:
            log.info(f"input {pos} on the target device")
            pos = self.check_value(pos)
            return air.wait(pos, timeout, interval, intervalfunc)
        except Exception as e:
            log.error(f"failed to input {pos} on the target device")
            raise e

    def wait_disappear_element(self, pos, *args, **kwargs):
        start_loading_time = time.time()
        while self.exists(pos) is not False:
            if time.time() - start_loading_time > 180:
                log.info(f"{pos}存在超时，请注意查看")
                return False
        log.info(f"{pos}消失")
        return True

    def wait_element_appear(self, pos, *args, **kwargs):
        start_loading_time = time.time()
        while self.exists(pos) is False:
            if time.time() - start_loading_time > 180:
                log.info(f"{pos}寻找超时，请注意查看")
                return False
        log.info(f"{pos}出现")
        return True

    def exists(self, pos, ocrPlus=None, **kwargs):
        try:
            log.info(f"the node: {pos} exists.")
            if ocrPlus is None:
                ocrPlus = False
            pos = self.check_value(pos, ocrPlus)
            return air.exists(pos)
        except Exception as e:
            log.error(f"the node: {pos} not exists.")
            raise e

    def persistent_element_exists(self, pos):
        pass

    def wait_for_any(self, pos_list: list, timeout=30):
        """

        :param pos_list: 需要等待的列表元素
        :param timeout: 超出时长
        :return:
        """
        start = time.time()
        while True:
            for pos in pos_list:
                if self.exists(pos):
                    return pos
            if time.time() - start > timeout:
                raise TargetNotFoundError(f'any to appear{pos_list}')

    def wait_for_all(self, pos_list: list, timeout=30):
        start = time.time()
        while True:
            all_exist = True
            for pos in pos_list:
                if not self.exists(pos):
                    all_exist = False
                    break
            if all_exist:
                return
            if time.time() - start > timeout:
                raise TargetNotFoundError(f'all to appear{pos_list}')

    def wait_next_element(self, last_click_pos, next_pos):
        """
        等待下一个元素出现
        :param last_click_pos: 需要点击的元素
        :param next_pos: 将会出现的元素
        :return:
        """
        start_time = time.time()
        while self.exists(next_pos) is False:
            log.info(f"{next_pos}没有出现正在尝试点击")
            self.click(last_click_pos)
            air.sleep(0.5)
            if time.time() - start_time > 180:
                log.info(f"{next_pos}没有出现")
                return False
        return True

    def get_text(self, pos):

        pass

    def check_value(self, val, ocrPlus=False):
        """
        用于判定是否是可以直接使用字典取值。如果不可以判断是否为文件夹路径是的话采用图片识别
        :param ocrPlus: 主要用于传递是否开启增强识别(不一定增强)
        :param val: 需要检查的参数
        :return:
        """
        if isinstance(val, str):
            dict_value = None
            if self.this_dict:
                dict_value = self.this_dict.get(val)
            if dict_value is not None:
                return dict_value
            log.info(f"字典中不存在{val}对应的路径")
            return myTemplate(val) if os.path.isfile(val) else OcrTemplate(val, language=self.language, ocrPlus=ocrPlus)
        return val


if __name__ == '__main__':
    air.sleep()



from airtestProject.commons.stateMachine.taskState import TaskEnd


class TaskMachine:
    def __init__(self, state_classes):
        self.states = {state_class.__name__: state_class() for state_class in state_classes}

    def run(self, test_cases):
        while not test_cases.empty():
            self.state = self.states['StartTest']  # 将状态设置为StartTest
            test_case = test_cases.get()
            try:
                while not isinstance(self.state, TaskEnd):  # 当状态不是TestEnd时，继续运行

                    self.state.run(test_case)
                    self.state = self.state.next_state()
            except Exception:
                self.state = self.states['TaskException']
                self.state.run(test_case)



import datetime
import os
import platform
import time

from airtestProject.airtest.cli.parser import cli_setup
from airtestProject.airtest.core.api import auto_setup
from airtestProject import config
from airtestProject.commons.utils.command import adb_shell
from airtestProject.commons.utils.reportUtil import ReportUtil
from airtestProject.commons.utils.tools import get_folder_path_up
from airtestProject.manager.LogManager import LogManager, catch_error

"""
author liuhuatong
des 设备状态管理控制
date 2024/4/29
"""

# state: no_connect/connect/occupation/free
DEVICE_INFO = {"320aa6ec": "小米mix2", "32883753": "小米11"}


class DeviceManager:
    def __init__(self):
        self.report_path = None
        self.logdir = None
        self.file_name = None
        self.file_path = None
        self.log_manager = LogManager("")

        self.devices = {}
        self.device_id = None
        self.get_devices()

    @catch_error
    def get_devices(self):
        """获取设备状态信息
        :return: dict()
        """
        self.devices = {}
        _str = os.popen("adb devices").readlines()
        _str.pop(0)
        _str.pop(-1)

        if _str:
            # todo 这里需要考虑一下，如果列表中没有设备的情况下，应该如何处理
            for i in _str:
                if 'device' in i:
                    _i = i.split('\t')[0]
                    if _i not in DEVICE_INFO:
                        self.devices.update({_i: {
                            "name": self.get_device_name(_i) if _i not in DEVICE_INFO else DEVICE_INFO[_i],
                            "state": "free"}})
                else:
                    self.devices.update({_i: {"id": _i, "name": self.get_device_name(_i) if _i not in DEVICE_INFO else
                    DEVICE_INFO[_i], "state": "no_connect"}})
        return self.devices

    def auto_setup(self, file_path=None, device_ids=None, logdir=False):
        """Airtest 设备连接
        :param file_path: 脚本文件目录
        :param logdir: 日志保存路径
        :param device_ids: 设备id
        :return: 返回日志保存路径
        """

        if not file_path:
            self.file_path = __file__
        else:
            self.file_path = file_path
        self.file_name = os.path.basename(self.file_path).split('.')[0]

        project_root = os.getcwd()  # 获取脚本运行路径

        if logdir is True:
            self.logdir = os.path.join(get_folder_path_up(self.file_path, "log"), r"{}-{}.log".format(self.file_name,
                                                                                                      datetime.datetime
                                                                                                      .now()
                                                                                                      .strftime(
                                                                                                          '%Y-%m-%d_'
                                                                                                          '%H-%M-%S')))
        self.report_path = os.path.join(get_folder_path_up(self.file_path, "reports"), r"reports.{}-{}"
                                        .format(self.file_name, datetime.datetime.now().strftime('%Y-%m-%d_%H-%M-%S')))
        if not device_ids:
            auto_setup(self.file_path, devices=[f"Android:///"], project_root=project_root, logdir=self.logdir)
            return self.logdir

        if isinstance(device_ids, str):
            device_ids = [device_ids]

        if not cli_setup():
            devices = []
            for i in device_ids:
                devices.append(f"android://127.0.0.1:5037/{i}?touch_method=MAXTOUCH&")
            auto_setup(self.file_path, devices=devices, project_root=project_root, logdir=self.logdir)
        return self.logdir

    def to_report(self):
        size = -1
        start_time = time.time()
        while time.time() - start_time < 60:
            if not os.path.exists(self.logdir):
                time.sleep(1)
                continue
            current_size = os.path.getsize(self.logdir)
            if current_size == size:
                break  # 文件大小没有变化，认为日志生成完毕
            else:
                size = current_size
                time.sleep(1)  # 等待一秒再次检查
        report = ReportUtil(script_root=self.file_path, log_root=self.logdir,
                            export_dir=self.report_path,
                            logfile=self.logdir + r"\log.txt",
                            lang='zh',
                            plugins=None)
        report.report()

    @catch_error
    def get_free_devices(self):
        """获取空闲设备信息
        :return: {}
        """
        self.get_devices()

        free_devices = {}
        if not self.devices:
            return {}

        for device_id, value in self.devices.items():
            if value["state"] == "free":
                free_devices.update({device_id: value})
        return free_devices

    @catch_error
    def get_device_name(self, device_id=None):
        """根据设备id获取设备名字
        :param device_id: 设备id
        :return: {}
        """
        devices_name = None
        try:

            devices_name = adb_shell("getprop ro.product.model", device_id)
        except:
            pass

        return devices_name

    @catch_error
    def get_sdk_version(self, device_id=None):
        """获取设备sdk版本号
        :param device_id: 设备id
        :return: str()
        """
        if device_id not in self.devices and device_id != None:
            self.log_manager.log_step("{} 设备没有连接!".format(device_id))
            return None
        api_verison = int(adb_shell("getprop ro.build.version.sdk", device_id))
        return api_verison

    @catch_error
    def get_runing_package(self, device_id=None):
        """获取运行中的包名字
        :param device_id: 设备id
        :return: str()
        """
        if device_id not in self.devices and device_id != None:
            self.log_manager.log_step("{} 设备没有连接!".format(device_id))
            return None

        packages_list = adb_shell("dumpsys window | findstr mCurrentFocus", device_id)
        if "com" not in packages_list:
            return ""
        package = packages_list.split("/")[0].split(" ")[-1]
        return package

    @catch_error
    def filter_type(self):
        """根据不同平台选择出pipe 过滤的方法
        :return: str()
        """
        """Select the pipe filtering method according to the system"""
        filter_type = ('grep', 'findstr')[platform.system() == config.PLATFORM["Windows"]]
        return filter_type

    @catch_error
    def get_pid(self, device_id, pkg_name):
        """获取设备运行中的pkg的pid
        :param device_id: 设备id
        :param pkg_name: 包名
        :return: str()
        """
        try:
            if device_id not in self.devices:
                self.log_manager.log_step("{}设备没有连接".format(device_id))
                return ""
            sdk_version = self.get_sdk_version(device_id)
            self.log_manager.log_step("filter", self.filter_type())
            self.log_manager.log_step(
                os.popen(f"adb -s {device_id} shell ps -ef | {self.filter_type()} {pkg_name}").readlines())
            if sdk_version and int(sdk_version) < 26:
                result = os.popen(f"adb -s {device_id} shell ps | {self.filter_type()} {pkg_name}").readlines()
                process_list = [{'{}'.format(process.split()[1]): '{}'.format(process.split()[8])} for process in
                                result]
            else:
                result = os.popen(f"adb -s {device_id} shell ps -ef | {self.filter_type()} {pkg_name}").readlines()
                process_list = [{'{}'.format(process.split()[1]): '{}'.format(process.split()[7])} for process in
                                result]
            if len(process_list) == 0:
                self.log_manager.log_step('{}: no pid found'.format(pkg_name))
        except Exception as e:
            process_list = []
            self.log_manager.log_error(e)
        return process_list

    @catch_error
    def reboot(self, device_id):
        """重启设备
        :param device_id: 设备id
        :return:
        """
        if device_id not in self.devices:
            self.log_manager.log_step("{} 设备没有连接!".format(device_id))
            return None

        result = adb_shell("reboot", device_id)
        self.log_manager.log_step("{} {} 设备重启\n{}".format("reboot", device_id, result))
        return True


if __name__ == "__main__":
    d = DeviceManager()

    print(d.get_runing_package())
    print(d.get_pid("d1236ad8", "com.global.bcslg"))
    #
    # print(d.get_runing_package("320aa6ec"))
    # print(d.get_pid("320aa6ec", "com.global.bcslg.test"))
    # print(d.get_sdk_version("320aa6ec"))
    # print(d.get_sdk_version("32883753"))



import os
import unittest
from datetime import datetime

from airtestProject.airtest.utils.logwraper import AirtestLogger

from airtestProject.manager.DeviceManager import DeviceManager
from airtestProject.manager.LogManager import LogManager


class TestCaseManager:
    def __init__(self, project):
        """
        初始化测试用例管理器
        :param project:
        """
        self.absolute_log_path = None
        logfile = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))+'\\log\\'+project+'\\log.txt'
        self.log_manager = LogManager(logfile=logfile, project=project)
        self.device_manager = DeviceManager(self.log_manager)
        # self.log_manager.log_step('连接设备')
        # todo: 这里的设备ID要根据实际情况修改
        self.app_start_case = self.device_manager.auto_setup()
        self.execution_date = datetime.now().strftime("%Y-%m-%d %H:%M:%S")
        self.executed_test_cases = []
        self.case_path = None
        self.current_directory = os.path.dirname(os.path.abspath(__file__))
        self.project_root = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))

    def load_tests_from_directory(self, directory, project):
        """
        加载测试套件
        :param directory:
        :return:
        """
        test_loader = unittest.TestLoader()
        suite = unittest.TestSuite()
        base_module = f'airtestProjects.cases.{project}'
        for filename in os.listdir(directory):
            if filename.endswith('.py'):
                module_name = filename[:-3]
                module_path = f"{base_module}.{module_name}"
                module = __import__(module_path, fromlist=['*'])
                test_class = getattr(module, module_name)
                test_class.setup(self.log_manager)
                loaded_tests = test_loader.loadTestsFromModule(module)
                suite.addTests(loaded_tests)
        return suite

    def run_tests(self, project):
        """
        运行测试用例，加载cases下面的项目的所有用例，并且执行
        :param project:
        :return:
        """
        # =====================================
        # 测试调用AirtestLogger的log来生成日志
        # log_path = self.current_directory + '\\log\\' + project + '\\log.txt'
        if not os.path.exists(self.project_root + '\\log\\' + project):
            os.makedirs(self.project_root + '\\log\\' + project)
        log_file = self.project_root + '\\log\\' + project + '\\log.txt'
        airtest_log = AirtestLogger(None)
        airtest_log.set_logfile(log_file)
        # =====================================

        test_suite = self.load_tests_from_directory(self.project_root + '/cases/' + project, project)
        for suite in test_suite:
            for test_case in suite:
                test_result = {'name': test_case.id(), 'start_time': datetime.now().strftime("%Y-%m-%d %H:%M:%S")}
                result = unittest.TextTestRunner().run(test_case)
                test_result['end_time'] = datetime.now().strftime("%Y-%m-%d %H:%M:%S")
                if result.wasSuccessful():
                    self.log_manager.log_step(f"测试通过： {test_case}")
                    test_result['status'] = "Passed"
                else:
                    self.log_manager.log_error(f"测试不通过：{test_case}")
                    test_result['status'] = "Failed"
                self.executed_test_cases.append(test_result)

        # 生成测试报告
        report_path = self.project_root + f"\\report\\{project}_{datetime.now().strftime('%Y-%m-%d-%H-%M')}.html"
        self.absolute_log_path = self.current_directory + '\\log\\all.txt'
        self.generate_report(output=report_path, project=project)

    def generate_report(self, output, project):
        """
         生成测试报告，包括设备信息、用例执行情况、执行日期等
        :param output:报告输出路径
        :param project:项目名称
        :return:NONE
        """
        self.log_manager.close_logger()
        # log根目录
        log_root = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
        # log的路径
        log_path = os.path.join(log_root, 'log\\' + project)
        # log文件
        log_file = os.path.join(log_path, f'log.txt')
        from airtestProject.manager.ReportManager import simple_report
        # from airtest.report.report import simple_report
        # 生成报告
        simple_report(__file__, logpath=log_path, logfile=log_file, output=output, project=project)


if __name__ == '__main__':
    project = 'zhange'
    test_case_manager = TestCaseManager(project=project)
    test_case_manager.run_tests(project)



# -*- coding: utf-8 -*-
from airtestProject.airtest import aircv
from airtestProject.airtest.core.device import Device
from pywinauto import mouse, keyboard
from Xlib import display, X
from PIL import Image
import socket
import time
import subprocess


class Linux(Device):
    """Linux desktop."""

    def __init__(self, pid=None, **kwargs):
        self.pid = None
        self._focus_rect = (0, 0, 0, 0)
        self.mouse = mouse
        self.keyboard = keyboard

    def shell(self, cmd):
        """
        Run shell command in subprocess

        Args:
            cmd: command to be run

        Raises:
            subprocess.CalledProcessError: when command returns non-zero exit status

        Returns:
            command output as a byte string

        """
        return subprocess.check_output(cmd, shell=True)

    def snapshot(self, filename="tmp.png", quality=None):
        """
        Take a screenshot and save it to `tmp.png` filename by default

        Args:
            filename: name of file where to store the screenshot
            quality: ignored

        Returns:
            display the screenshot

        """
        w, h = self.get_current_resolution()
        dsp = display.Display()
        root = dsp.screen().root
        raw = root.get_image(0, 0, w, h, X.ZPixmap, 0xffffffff)
        image = Image.frombytes("RGB", (w, h), raw.data, "raw", "BGRX")
        from airtest.aircv.utils import pil_2_cv2
        image = pil_2_cv2(image)
        return image

    def keyevent(self, keyname, **kwargs):
        """
        Perform a key event

        References:
            https://pywinauto.readthedocs.io/en/latest/code/pywinauto.keyboard.html

        Args:
            keyname: key event
            **kwargs: optional arguments

        Returns:
            None

        """
        self.keyboard.SendKeys(keyname)

    def text(self, text, **kwargs):
        """
        Input text

        Args:
            text: text to input
            **kwargs: optional arguments

        Returns:
            None

        """
        self.keyevent(text)

    def touch(self, pos, **kwargs):
        """
        Perform mouse click action

        References:
            https://pywinauto.readthedocs.io/en/latest/code/pywinauto.mouse.html

        Args:
            pos: coordinates where to click
            **kwargs: optional arguments

        Returns:
            None

        """
        duration = kwargs.get("duration", 0.01)
        right_click = kwargs.get("right_click", False)
        button = "right" if right_click else "left"

        self.mouse.press(button=button, coords=pos)
        time.sleep(duration)
        self.mouse.release(button=button, coords=pos)

    def double_click(self, pos):
        self.mouse.double_click(coords=pos)

    def swipe(self, p1, p2, duration=0.8, steps=5):
        """
        Perform swipe (mouse press and mouse release)
        Args:
            p1: start point
            p2: end point
            duration: time interval to perform the swipe action
            steps: size of the swipe step

        Returns:
            None

        """
        from_x, from_y = p1
        to_x, to_y = p2

        interval = float(duration) / (steps + 1)
        self.mouse.press(coords=(from_x, from_y))
        time.sleep(interval)
        for i in range(1, steps):
            self.mouse.move(coords=(
                int(from_x + (to_x - from_x) * i / steps),
                int(from_y + (to_y - from_y) * i / steps),
            ))
            time.sleep(interval)
        for i in range(10):
            self.mouse.move(coords=(to_x, to_y))
        time.sleep(interval)
        self.mouse.release(coords=(to_x, to_y))

    def start_app(self, path, *args, **kwargs):
        """
        Start the application

        Args:
            path: full path to the application

        Returns:
            None

        """
        super(Linux, self).start_app(path)

    def stop_app(self, pid):
        """
        Stop the application

        Args:
            pid: process ID of the application to be stopped

        Returns:
            None

        """
        super(Linux, self).stop_app(pid)

    def get_current_resolution(self):
        d = display.Display()
        screen = d.screen()
        w, h = (screen["width_in_pixels"], screen["height_in_pixels"])
        return w, h

    def get_ip_address(self):
        """
        Return default external ip address of the linux os.

        Returns:
             :py:obj:`str`: ip address
        """
        return socket.gethostbyname(socket.gethostname())



# coding=utf-8
from __future__ import division
from __future__ import print_function
from __future__ import absolute_import

import cv2
import ffmpeg
import threading
import time
import numpy as np
import subprocess


RECORDER_ORI = {
    "PORTRAIT": 1,
    "LANDSCAPE": 2,
    "ROTATION": 0,  # The screen is centered in a square
}

def resize_by_max(img, max_size=800):
    if img is None:
        return np.zeros((max_size, max_size, 3), dtype=np.uint8)
    max_len = max(img.shape[0], img.shape[1])
    if max_len > max_size:
        scale = max_size / max_len
        img = cv2.resize(img, (int(img.shape[1] * scale), int(img.shape[0] * scale)))
    return img


def get_max_size(max_size):
    try:
        max_size = int(max_size)
    except:
        max_size = None
    else:
        if max_size <= 0:
            max_size = None
    return max_size


class FfmpegVidWriter:
    """
    Generate a video using FFMPEG.
    """
    def __init__(self, outfile, width, height, fps=10, orientation=0):
        self.fps = fps

        # 三种横竖屏录屏模式 1 竖屏 2 横屏 0 方形居中
        self.orientation = RECORDER_ORI.get(str(orientation).upper(), orientation)
        if self.orientation == 1:
            self.height = max(width, height)
            self.width = min(width, height)
        elif self.orientation == 2:
            self.width = max(width, height)
            self.height = min(width, height)
        else:
            self.width = self.height = max(width, height)

        # 满足视频宽高条件
        self.height = height = self.height - (self.height % 32) + 32
        self.width = width = self.width - (self.width % 32) + 32
        self.cache_frame = np.zeros((height, width, 3), dtype=np.uint8)

        try:
            subprocess.Popen("ffmpeg", stdout=subprocess.DEVNULL, stderr=subprocess.DEVNULL).wait()
        except FileNotFoundError:
            from airtest.utils.ffmpeg import ffmpeg_setter
            try:
                ffmpeg_setter.add_paths()
            except Exception as e:
                print("Error: setting ffmpeg path failed, please download it at https://ffmpeg.org/download.html then add ffmpeg path to PATH")
                raise

        self.process = (
            ffmpeg
            .input('pipe:', format='rawvideo', pix_fmt='rgb24',
                s='{}x{}'.format(width, height), framerate=self.fps)
            .output(outfile, pix_fmt='yuv420p', vcodec='libx264', crf=25,
                    preset="veryfast", framerate=self.fps)
            .global_args("-loglevel", "error")
            .overwrite_output()
            .run_async(pipe_stdin=True)
        )
        self.writer = self.process.stdin

    def process_frame(self, frame):
        assert len(frame.shape) == 3
        frame = frame[..., ::-1]
        if self.orientation == 1 and frame.shape[1] > frame.shape[0]:
            frame = cv2.resize(frame, (self.width, int(self.width*self.width/self.height)))
        elif self.orientation == 2 and frame.shape[1] < frame.shape[0]:
            frame = cv2.resize(frame, (int(self.height*self.height/self.width), self.height))
        h_st = max(self.cache_frame.shape[0]//2 - frame.shape[0]//2, 0)
        w_st = max(self.cache_frame.shape[1]//2 - frame.shape[1]//2, 0)
        h_ed = min(h_st+frame.shape[0], self.cache_frame.shape[0])
        w_ed = min(w_st+frame.shape[1], self.cache_frame.shape[1])
        self.cache_frame[:] = 0
        self.cache_frame[h_st:h_ed, w_st:w_ed, :] = frame[:(h_ed-h_st), :(w_ed-w_st)]
        return self.cache_frame.copy()

    def write(self, frame):
        self.writer.write(frame.astype(np.uint8))

    def close(self):
        self.writer.close()
        self.process.wait()
        self.process.terminate()


class ScreenRecorder:
    def __init__(self, outfile, get_frame_func, fps=10, snapshot_sleep=0.001, orientation=0):
        self.get_frame_func = get_frame_func
        self.tmp_frame = self.get_frame_func()
        self.snapshot_sleep = snapshot_sleep

        width, height = self.tmp_frame.shape[1], self.tmp_frame.shape[0]
        self.writer = FfmpegVidWriter(outfile, width, height, fps, orientation)
        self.tmp_frame = self.writer.process_frame(self.tmp_frame)

        self._is_running = False
        self._stop_flag = False
        self._stop_time = 0

    def is_running(self):
        return self._is_running

    @property
    def stop_time(self):
        return self._stop_time

    @stop_time.setter
    def stop_time(self, max_time):
        if isinstance(max_time, int) and max_time > 0:
            self._stop_time = time.time() + max_time
        else:
            print("failed to set stop time")

    def is_stop(self):
        if self._stop_flag:
            return True
        if self._stop_time > 0 and time.time() >= self._stop_time:
            return True
        return False

    def start(self):
        if self._is_running:
            print("recording is already running, please don't call again")
            return False
        self._is_running = True
        self.t_stream = threading.Thread(target=self.get_frame_loop)
        self.t_stream.setDaemon(True)
        self.t_stream.start()
        self.t_write = threading.Thread(target=self.write_frame_loop)
        self.t_write.setDaemon(True)
        self.t_write.start()
        return True

    def stop(self):
        self._is_running = False
        self._stop_flag = True
        self.t_write.join()
        self.t_stream.join()

    def get_frame_loop(self):
        # 单独一个线程持续截图
        try:
            while True:
                tmp_frame = self.get_frame_func()
                self.tmp_frame = self.writer.process_frame(tmp_frame)
                time.sleep(self.snapshot_sleep)
                if self.is_stop():
                    break
            self._stop_flag = True
        except Exception as e:
            print("record thread error", e)
            self._stop_flag = True
            raise

    def write_frame_loop(self):
        # 按帧率间隔获取图像写入视频
        try:
            duration = 1.0/self.writer.fps
            last_time = time.time()
            self._stop_flag = False
            while True:
                if time.time()-last_time >= duration:
                    last_time += duration
                    self.writer.write(self.tmp_frame)
                if self.is_stop():
                    break
                time.sleep(0.0001)
            self.writer.close()
            self._stop_flag = True
        except Exception as e:
            print("write thread error", e)
            self._stop_flag = True
            raise
